#! /usr/bin/env python

import openturns as ot
import persalys

myStudy = persalys.Study("myStudy")

X0 = persalys.Input("X0", ot.Normal(1, 1))
X1 = persalys.Input("X1", ot.Normal(1, 1))
Y00 = persalys.Output("fake_Y0")
Y00.setIsSelected(False)
Y0 = persalys.Output("Y0")

formula_Y00 = "X0"
formula_Y0 = "sin(X0) + 8*X1"
model = persalys.SymbolicPhysicalModel(
    "aModelPhys", [X0, X1], [Y00, Y0], [formula_Y00, formula_Y0]
)
myStudy.add(model)

# Taylor Expansion ##
analysis = persalys.TaylorExpansionMomentsAnalysis("aTaylor", model)
myStudy.add(analysis)
print(analysis)

analysis.run()

print("result=", analysis.getResult())
print("\n")

# Taylor Expansion ##
# test with a deterministic variable
X2 = persalys.Input("X2", 10)
model.addInput(X2)
model.addOutput(persalys.Output("Y1"))
model.setFormula("Y1", "sin(X0) + 8*X1 + X2")

analysis2 = persalys.TaylorExpansionMomentsAnalysis("aTaylor2", model)
analysis2.setInterestVariables(["Y1"])
myStudy.add(analysis2)
print(analysis2)

analysis2.run()

print("result=", analysis2.getResult())
print("\n")

# Taylor Expansion ##
# test definition gradient with PythonPhysicalModel
code = "from math import sin\n\ndef _exec(X0, X1):\n    Y0 = sin(X0) + 8*X1\n    fake_Y0 = X0\n    return Y0, fake_Y0\n"
model2 = persalys.PythonPhysicalModel("myPhysicalModel", [X0, X1], [Y0], code)
model2.setProcessNumber(1)
myStudy.add(model2)

# default finite difference steps
analysis3 = persalys.TaylorExpansionMomentsAnalysis("aTaylor3", model2)
analysis3.setInterestVariables(["Y0"])
myStudy.add(analysis3)
print(analysis3)

analysis3.run()

print("result=", analysis3.getResult())
print("\n")

# greater finite difference steps
model2.setFiniteDifferenceStep("X0", 0.005)
model2.setFiniteDifferenceStep("X1", 0.005)

analysis4 = persalys.TaylorExpansionMomentsAnalysis("aTaylor4", model2)
analysis4.setInterestVariables(["Y0"])
myStudy.add(analysis4)
print(analysis4)

analysis4.run()

print("result=", analysis4.getResult())

# script
script = myStudy.getPythonScript()
print(script)
exec(script)



#! /usr/bin/env python

import openturns as ot
import persalys

myStudy = persalys.Study("myStudy")

# Model
X0 = persalys.Input("X0", ot.Normal(1, 1))
X1 = persalys.Input("X1", ot.Normal(1, 1))
Y0 = persalys.Output("Y0")

model = persalys.SymbolicPhysicalModel("aModelPhys", [X0, X1], [Y0], ["sin(X0) + 8*X1"])
myStudy.add(model)

# limit state ##
limitState = persalys.LimitState("aLimitState", model, "Y0", ot.Greater(), 20.0)
print(limitState)
myStudy.add(limitState)

limitState.setThreshold(10.0)
limitState.setOperator(ot.Less())
print(limitState)

# limit state ##
limitState2 = persalys.LimitState("aLimitState2", model)
print(limitState2)
myStudy.add(limitState2)

limitState2.setThreshold(15.0)
print(limitState2)

model.addOutput(persalys.Output("Y1"))
model.setFormula("Y1", "1 + sin(X0) + 8*X1")
limitState2.setOutputName("Y1")
print(limitState2)

# script
script = myStudy.getPythonScript()
print(script)
exec(script)



#! /usr/bin/env python

import openturns as ot
import persalys
import os

g = ot.SymbolicFunction(["x1", "x2"], ["x1+x2", "3*(x1+x2)", "x1*x2"])
f = persalys.FileMemoizeFunction(g, "in_fm.csv", "out_fm.csv")
for i in range(5):
    f([1, 2])
    f([10 + i, 20 + i])
print("f.in", f.getCacheInput())
print("f.out", f.getCacheOutput())
print("f.hits", f.getCacheHits())
print("f.calls", f.getCallsNumber())
print("f.evals", f.getEvaluationCallsNumber())
g2 = ot.SymbolicFunction(["x1", "x2"], ["?"] * 3)
f2 = persalys.FileMemoizeFunction(g2, "in_fm.csv", "out_fm.csv")
for i in range(5):
    f2([1, 2])
    f2([10 + i, 20 + i])
print("f2.in", f2.getCacheInput())
print("f2.out", f2.getCacheOutput())
print("f2.hits", f2.getCacheHits())
print("f2.calls", f2.getCallsNumber())
print("f2.evals", f2.getEvaluationCallsNumber())

os.remove(f.getCacheInputFile())
os.remove(f.getCacheOutputFile())



#!/usr/bin/env python

import openturns as ot
import openturns.testing
import persalys

ot.RandomGenerator.SetSeed(0)

myStudy = persalys.Study("myStudy")
persalys.Study.Add(myStudy)

# Models
x1 = persalys.Input("x1", 0.2, ot.Uniform(0.0, 10.0))
x2 = persalys.Input("x2", 1.2, ot.Uniform(0.0, 10.0))
x3 = persalys.Input("x3", 1.0)
fake_var = persalys.Output("fake_var")
fake_var.setIsSelected(False)
y0 = persalys.Output("y0")
fake_y0 = persalys.Output("fake_y0")
y1 = persalys.Output("y1")

# model 1 ##
formula_fake_var = "x1"
formula_y0 = "cos(0.5*x1) + sin(x2)"
formula_y1 = "cos(0.5*x1) + sin(x2) + x3"
model1 = persalys.SymbolicPhysicalModel(
    "model1",
    [x1, x2, x3],
    [fake_var, y0, fake_y0, y1],
    [formula_fake_var, formula_y0, formula_y0, formula_y1],
)
R = ot.CorrelationMatrix(2)
R[0, 1] = 0.25
model1.setCopula(["x1", "x2"], ot.NormalCopula(R))
myStudy.add(model1)

# model 3 ##
filename = "data_pa.csv"
cDist = ot.JointDistribution(
    [ot.Normal(), ot.Gumbel(), ot.Normal(), ot.Uniform()],
    ot.BlockIndependentCopula([ot.IndependentCopula(2), ot.GumbelCopula()]),
)
sample = cDist.getSample(200)
sample.exportToCSVFile(filename, " ")
model3 = persalys.DataModel(
    "model3", filename, [0, 2, 3], [1], ["x_0", "x_2", "x_3"], ["x_1"]
)
myStudy.add(model3)

# Designs of Experiment ##

# design 2 ##
values = [[0.5 + i * 1.5 for i in range(7)], [0.5 + i * 1.5 for i in range(7)], [1]]
design_2 = persalys.GridDesignOfExperiment("design_2", model1, values)
myStudy.add(design_2)

# design 4 ##
probaDesign = persalys.ProbabilisticDesignOfExperiment(
    "probaDesign", model1, 100, "MONTE_CARLO"
)
probaDesign.run()
myStudy.add(probaDesign)

design_3 = persalys.ImportedDesignOfExperiment("design_3", model1, filename, [0, 2, 3])
design_3.run()
myStudy.add(design_3)


# 1- meta model1 ##

# 1-a Kriging ##
kriging = persalys.KrigingAnalysis("kriging", probaDesign)
kriging.setBasis(ot.LinearBasisFactory(2).build())
kriging.setCovarianceModel(ot.MaternModel(2))
kriging.setTestSampleValidation(True)
kriging.setKFoldValidation(True)
kriging.setInterestVariables(["y0", "y1"])
myStudy.add(kriging)

# 1-b Chaos ##
chaos1 = persalys.FunctionalChaosAnalysis("chaos_1", probaDesign)
chaos1.setChaosDegree(7)
chaos1.setSparseChaos(True)
chaos1.setTestSampleValidation(True)
chaos1.setKFoldValidation(True)
chaos1.setInterestVariables(["y1"])
myStudy.add(chaos1)

# 1-c Chaos ##
chaos2 = persalys.FunctionalChaosAnalysis("chaos_2", design_2)
chaos2.setChaosDegree(2)
chaos2.setSparseChaos(True)
myStudy.add(chaos2)

# 1-d linear regression #
linreg = persalys.PolynomialRegressionAnalysis("linreg", probaDesign)
linreg.setInterestVariables(["y0", "y1"])
linreg.setDegree(2)
linreg.setInteraction(False)
linreg.setTestSampleValidation(True)
linreg.setKFoldValidation(True)
myStudy.add(linreg)

# 2- central tendancy ##

# 2-a Monte Carlo ##
monteCarlo = persalys.MonteCarloAnalysis("MonteCarlo", model1)
monteCarlo.setIsConfidenceIntervalRequired(False)
monteCarlo.setMaximumCoefficientOfVariation(-1.0)
monteCarlo.setMaximumElapsedTime(1000)
monteCarlo.setMaximumCalls(1000)
monteCarlo.setBlockSize(100)
monteCarlo.setSeed(2)
monteCarlo.setInterestVariables(["y0", "y1"])
myStudy.add(monteCarlo)

# 2-b Taylor Expansion ##
taylor = persalys.TaylorExpansionMomentsAnalysis("Taylor", model1)
taylor.setInterestVariables(["y0", "y1"])
myStudy.add(taylor)

# 3- reliability ##

# limit state ##
limitState = persalys.LimitState("aLimitState", model1, "y1", ot.Greater(), 0.5)
myStudy.add(limitState)

optimAlgo = ot.AbdoRackwitz()
optimAlgo.setMaximumIterationNumber(150)
optimAlgo.setMaximumAbsoluteError(1e-3)

# 3-a Monte Carlo ##
monteCarloReliability = persalys.MonteCarloReliabilityAnalysis(
    "MonteCarloReliability", limitState
)
monteCarloReliability.setMaximumCoefficientOfVariation(-1.0)
monteCarloReliability.setMaximumElapsedTime(1000)
monteCarloReliability.setMaximumCalls(1000)
monteCarloReliability.setBlockSize(100)
monteCarloReliability.setSeed(2)
myStudy.add(monteCarloReliability)

# 3-b FORM IS ##
form_is = persalys.FORMImportanceSamplingAnalysis("FORM_IS", limitState)
form_is.setOptimizationAlgorithm(optimAlgo)
form_is.setMaximumCoefficientOfVariation(-1.0)
form_is.setMaximumElapsedTime(1000)
form_is.setMaximumCalls(1000)
form_is.setBlockSize(100)
form_is.setSeed(2)
myStudy.add(form_is)

# 3-c FORM ##
form = persalys.FORMAnalysis("FORM", limitState)
form.setOptimizationAlgorithm(optimAlgo)
myStudy.add(form)

# 3-d SORM ##
sorm = persalys.SORMAnalysis("SORM", limitState)
sorm.setOptimizationAlgorithm(optimAlgo)
myStudy.add(sorm)

# 4- sensitivity ##

# 4-a Sobol ##
sobol = persalys.SobolAnalysis("Sobol", model1)
sobol.setMaximumConfidenceIntervalLength(-1)
sobol.setMaximumElapsedTime(1000)
sobol.setMaximumCalls(1000)
sobol.setReplicationSize(100)
sobol.setBlockSize(100)
sobol.setSeed(2)
sobol.setInterestVariables(["y0", "y1"])
myStudy.add(sobol)

# 4-b SRC ##
src = persalys.SRCAnalysis("SRC", model1)
src.setSimulationsNumber(200)
src.setSeed(2)
src.setInterestVariables(["y0", "y1"])
myStudy.add(src)

# 7- data analysis ##
dataAnalysis = persalys.DataAnalysis("DataAnalysis", model3)
myStudy.add(dataAnalysis)

# 8- Marginals inference ##
inference = persalys.InferenceAnalysis("inference", model3)
inference.setInterestVariables(["x_0", "x_3"])
factories = [ot.NormalFactory(), ot.GumbelFactory()]
inference.setDistributionsFactories("x_3", factories)
inference.setLevel(0.1)
myStudy.add(inference)

# 9- Copula inference ##
copulaInference = persalys.CopulaInferenceAnalysis("copulaInference", model3)
factories = [ot.NormalCopulaFactory(), ot.GumbelCopulaFactory()]
copulaInference.setDistributionsFactories(["x_0", "x_3"], factories)
myStudy.add(copulaInference)



#! /usr/bin/env python

import persalys
import openturns as ot
import openturns.testing
import math

sample = ot.Sample(0, 3)
sample.add([4, 2, 4])
sample.add([2, math.nan, 4])
sample.add([2, 3, 7])
sample.add([8, 7, 3])
sample.add([math.inf, math.nan, 7])


clean = persalys.DataCleaning(sample)
openturns.testing.assert_almost_equal(clean.getMean(), [4, 4, 5])
openturns.testing.assert_almost_equal(clean.getMedian(), [3, 3, 4])

# remove points containing Nans/Infs
clean.removeAllNans()
openturns.testing.assert_almost_equal(
    clean.getSample(), [[4, 2, 4], [2, 3, 7], [8, 7, 3]]
)

clean = persalys.DataCleaning(sample)
# replace Nans/Infs with zeros
clean.replaceAllNans([0, 0, 0])
openturns.testing.assert_almost_equal(
    clean.getSample(), [[4, 2, 4], [2, 0, 4], [2, 3, 7], [8, 7, 3], [0, 0, 7]]
)

clean = persalys.DataCleaning(sample)
# replace Nans/Infs with medians
clean.replaceAllNans(clean.getMedian())
openturns.testing.assert_almost_equal(
    clean.getSample(), [[4, 2, 4], [2, 3, 4], [2, 3, 7], [8, 7, 3], [3, 3, 7]]
)

clean.computeGeometricMAD()
openturns.testing.assert_almost_equal(clean.getMAD(), [1, 0, 1])
openturns.testing.assert_almost_equal(clean.getGeometricMAD(), 1.41421)

clean = persalys.DataCleaning(sample)
# replace Nans/Infs with means
clean.replaceAllNans(clean.getMean())
openturns.testing.assert_almost_equal(
    clean.getSample(), [[4, 2, 4], [2, 4, 4], [2, 3, 7], [8, 7, 3], [4, 4, 7]]
)

# compute MAD
clean.computeGeometricMAD()
openturns.testing.assert_almost_equal(clean.getMAD(), [1, 1, 1])
openturns.testing.assert_almost_equal(clean.getGeometricMAD(), 1.73205)

clean = persalys.DataCleaning(sample)
# replace Nans by column
clean.replaceNansByColumn(0, 0)
clean.replaceNansByColumn(1, 0)
openturns.testing.assert_almost_equal(
    clean.getSample(), [[4, 2, 4], [2, 0, 4], [2, 3, 7], [8, 7, 3], [0, 0, 7]]
)

clean = persalys.DataCleaning(sample)
# remove point / replace values column by column
clean.removeNansByColumn(0)
clean.replaceNansByColumn(1, -2)
openturns.testing.assert_almost_equal(
    clean.getSample(), [[4, 2, 4], [2, -2, 4], [2, 3, 7], [8, 7, 3]]
)



#!/usr/bin/env python

import openturns as ot
import openturns.testing
import persalys

ot.RandomGenerator.SetSeed(0)
myStudy = persalys.Study("myStudy")

# Model
xi1 = persalys.Input("xi1", ot.Uniform(0.0, 10.0))
xi2 = persalys.Input("xi2", ot.Uniform(0.0, 10.0))
xi3 = persalys.Input("xi3", 0.5)
y00 = persalys.Output("fake_y0")
y00.setIsSelected(False)
y0 = persalys.Output("y0")

formula_y00 = "xi1"
formula_y0 = "cos(0.5*xi1) + sin(xi2)"
model = persalys.SymbolicPhysicalModel(
    "model", [xi1, xi2, xi3], [y00, y0], [formula_y00, formula_y0]
)
myStudy.add(model)

# Design of Experiment ##
aDesign = persalys.FixedDesignOfExperiment("design", model)
validationInputSample = ot.LHSExperiment(model.getDistribution(), 10).generate()
inputSample = ot.Sample(validationInputSample)
inputSample.stack(ot.Sample(10, [0.5]))
aDesign.setOriginalInputSample(inputSample)
myStudy.add(aDesign)

aDesign.run()

# Kriging ##
analysis = persalys.KrigingAnalysis("kriging_0", aDesign)
analysis.setBasis(ot.LinearBasisFactory(2).build())
analysis.setCovarianceModel(ot.MaternModel(2))
myStudy.add(analysis)
print(analysis)

analysis.run()
metaModel = analysis.getResult().getResultForVariable("y0").getMetaModel()
openturns.testing.assert_almost_equal(
    aDesign.getResult().getDesignOfExperiment().getOutputSample(),
    metaModel(validationInputSample),
    3.0e-5,
    3.0e-5,
)

# Design of Experiment ##
model.addOutput(persalys.Output("y1"))
model.setFormula("y1", formula_y0 + " + xi3")
aDesign.setInterestVariables(["y0", "y1"])
aDesign.run()

# Kriging ##
analysis2 = persalys.KrigingAnalysis("kriging_1", aDesign)
analysis2.setAnalyticalValidation(True)
analysis2.setInterestVariables(["y1", "y0"])
myStudy.add(analysis2)
print(analysis2)

analysis2.run()

result2 = analysis2.getResult()
metaModel2 = result2.getResultForVariable("y1").getMetaModel()
openturns.testing.assert_almost_equal(
    aDesign.getResult().getDesignOfExperiment().getOutputSample().getMarginal(1),
    metaModel2(validationInputSample),
    3.0e-5,
    3.0e-5,
)
openturns.testing.assert_almost_equal(
    result2.getAnalyticalValidation().getQ2(), [0.685791] * 2, 1e-3, 1e-3
)

# Kriging ##
analysis3 = persalys.KrigingAnalysis("kriging_2", aDesign)
analysis3.setOptimizeParameters(False)
analysis3.setInterestVariables(["y0"])
myStudy.add(analysis3)
print(analysis3)

analysis3.run()

result3 = analysis3.getResult()
# print("result=", result3)

metaModel3 = result3.getResultForVariable("y0").getMetaModel()
openturns.testing.assert_almost_equal(
    aDesign.getResult().getDesignOfExperiment().getOutputSample().getMarginal(0),
    metaModel3(validationInputSample),
    3.0e-5,
    3.0e-5,
)

# script
script = myStudy.getPythonScript()
print(script)
exec(script)



#!/usr/bin/env python

import os
import sys
import persalys

# import openturns as ot
import openturns.testing as ott

ssh_hostname = ""

with open("input_logistic.txt.in", "w") as f:
    f.write("y0=@y0@\n")
    f.write("a=@a@\n")
    f.write("b=@b@\n")

with open("program_logistic.py", "w") as f:
    f.write("import math as m\n")
    f.write("def logisticSolution(t, y0, a, b):\n")
    f.write("    t0 = 1790.0\n")
    f.write("    y = a * y0 / (b * y0 + (a - b * y0) * m.exp(-a * (t - t0)))\n")
    f.write("    return y\n\n")
    f.write('exec(open("input_logistic.txt").read())\n')
    f.write("Y = [logisticSolution(t, y0, a, b) for t in range(1790, 2011)]\n")
    f.write('with open("Y.txt", "w") as f:\n')
    f.write("    for y in Y:\n")
    f.write('        f.write("%.17g\\n" % y)\n')

with open("post_logistic.py", "w") as f:
    f.write("import math as m\n")
    f.write('with open("Y.txt") as f:\n')
    f.write("    Y = [float(line) for line in f.readlines()]\n")
    f.write("Y_min = min(Y)\n")
    f.write("Y_max = max(Y)\n")
    f.write("Y_mean = sum(Y) / len(Y)\n")
    f.write("Y_last = Y[-1]\n")
    f.write("try:\n    idx = next(i for i,y in enumerate(Y) if y > 100e6)\n")
    f.write(
        "    Y_mean_t = sum(Y[:idx]) / idx\nexcept StopIteration:\n    Y_mean_t = 0.0\n"
    )
    f.write('with open("output.txt", "w") as f:\n')
    f.write('    f.write("Y_min=%.17g\\n" % Y_min)\n')
    f.write('    f.write("Y_max=%.17g\\n" % Y_max)\n')
    f.write('    f.write("Y_mean=%.17g\\n" % Y_mean)\n')
    f.write('    f.write("Y_last=%.17g\\n" % Y_last)\n')
    f.write('    f.write("Y_mean_t=%.17g\\n" % Y_mean_t)\n')


input_file1 = persalys.CouplingInputFile("input_logistic.txt.in")
input_file1.setConfiguredPath("input_logistic.txt")
input_file1.setVariables(
    ["y0", "a", "b"], ["@y0@", "@a@", "@b@"], ["{:10.6f}", "{:10.6f}", "{:10.6f}"]
)
resource_file1 = persalys.CouplingResourceFile("program_logistic.py")
step1 = persalys.CouplingStep(
    sys.executable + " program_logistic.py", [input_file1], [resource_file1], []
)

resource_file2 = persalys.CouplingResourceFile("post_logistic.py")
output_file2 = persalys.CouplingOutputFile("output.txt")
output_file2.setVariables(
    ["Y_min", "Y_max", "Y_mean", "Y_last", "Y_mean_t"],
    ["Y_min=", "Y_max=", "Y_mean=", "Y_last=", "Y_mean_t="],
    [0] * 5,
    [0] * 5,
    [0] * 5,
)
step2 = persalys.CouplingStep(
    sys.executable + " post_logistic.py", [], [resource_file2], [output_file2]
)

model = persalys.CouplingPhysicalModel("logistic", [step1, step2])
model.setSSHHostname(ssh_hostname)
print(model.getInputNames())
print(model.getOutputNames())
assert model.getInputNames() == ["y0", "a", "b"]
assert model.getOutputNames() == ["Y_min", "Y_max", "Y_mean", "Y_last", "Y_mean_t"]

# single evaluation
x = [3.9e6, 0.03134, 1.5887e-10]
f = model.getFunction()
y = f(x)
print(y)
ott.assert_almost_equal(y, [3.9e06, 3.8498e09, 5.64037e08, 3.8498e09, 2.94863e07])

# cleanup
os.remove("input_logistic.txt.in")
os.remove("program_logistic.py")
os.remove("post_logistic.py")

# script
myStudy = persalys.Study("myStudy")
myStudy.add(model)
script = myStudy.getPythonScript()
# print('script=', script)
exec(script)



#!/usr/bin/env python


import persalys


def test():

    Study_0 = persalys.Study("Study_0")
    persalys.Study.Add(Study_0)
    X0 = persalys.Input("X0", 0, "")
    Y0 = persalys.Output("Y0", "")
    Y0.setValue(0)
    inputCollection = []
    inputCollection.append(X0)
    outputCollection = []
    outputCollection.append(Y0)
    code = "def _exec(X0):\n    Y0 = X0\n    return Y0"
    PhysicalModel_0 = persalys.PythonPhysicalModel(
        "PhysicalModel_0", inputCollection, outputCollection, code
    )
    PhysicalModel_0.setProcessNumber(1)
    Study_0.add(PhysicalModel_0)
    print(Study_0.getPythonScript())


test()



#!/usr/bin/env python

import openturns as ot
import openturns.testing as ott
import persalys

ot.RandomGenerator.SetSeed(0)

myStudy = persalys.Study("myStudy")
persalys.Study.Add(myStudy)

R = persalys.Input(
    "R", 700e6, ot.LogNormalMuSigma(750e6, 11e6).getDistribution(), "Parameter R"
)
C = persalys.Input("C", 2500e6, ot.Normal(2750e6, 250e6), "Parameter C")
gamma = persalys.Input("gam", 8.0, ot.Normal(10, 2), "Parameter gamma")
dist_strain = ot.Uniform(0, 0.07)
strain = persalys.Input("strain", 0.0, dist_strain, "Strain")
sigma = persalys.Output("sigma", "Stress (Pa)")
fakeSigma = persalys.Output("fakeSigma", "Stress (Pa)")

formula = ["R + C * (1 - exp(-gam * strain))", "2*(R + C * (1 - exp(-gam * strain)))"]
model = persalys.SymbolicPhysicalModel(
    "model1", [R, C, gamma, strain], [sigma, fakeSigma], formula
)
myStudy.add(model)

# generate observations
nbObs = 100
strainObs = dist_strain.getSample(nbObs)
strainObs.setDescription(["strain"])

stressSampleNoise = ot.Normal(0.0, 40.0e6).getSample(nbObs)
stressSample = ot.ParametricFunction(
    model.getFunction("sigma"), [0, 1, 2], [750e6, 2750e6, 10.0]
)(strainObs)

stressObs = stressSample + stressSampleNoise
stressObs.setDescription(["sigma"])
observations = persalys.Observations("obs1", model, strainObs, stressObs)

myStudy.add(observations)

# Least Squares linear
analysis = persalys.CalibrationAnalysis("myAnalysis", observations)
analysis.run()
myStudy.add(analysis)
print("analysis=", analysis)
thetaMAP = analysis.getResult().getCalibrationResult().getParameterMAP()
ott.assert_almost_equal(thetaMAP, [7.47009e+08, 2.73521e+09, 10.1291], 1e-2, 1)
print("isBayesian=", analysis.getResult().getCalibrationResult().isBayesian())

# Least Squares Non linear
analysis2 = persalys.CalibrationAnalysis("myAnalysis2", observations)
analysis2.setMethodName("LeastSquaresNonlinear")
analysis2.setCalibratedInputs(["R", "C"], ot.Dirac([700e6, 2400e6]), ["gam"], [7.0])
analysis2.run()
myStudy.add(analysis2)
print("analysis=", analysis2)
print("isBayesian=", analysis2.getResult().getCalibrationResult().isBayesian())

# Gaussian linear calibration
analysis3 = persalys.CalibrationAnalysis("myAnalysis3", observations)
sigmaStress = 1.0e7  # (Pa)
errorCovariance = ot.CovarianceMatrix(1)
errorCovariance[0, 0] = sigmaStress**2
analysis3.setMethodName("GaussianLinear")
analysis3.setErrorCovariance(errorCovariance)
analysis3.run()
myStudy.add(analysis3)
print("analysis=", analysis3)
print("isBayesian=", analysis3.getResult().getCalibrationResult().isBayesian())

# Gaussian non linear calibration
analysis4 = persalys.CalibrationAnalysis("myAnalysis4", observations)
analysis4.setMethodName("GaussianNonlinear")
analysis4.setErrorCovariance(errorCovariance)
analysis4.setBootStrapSize(50)
analysis4.setConfidenceIntervalLength(0.99)
optimizationAlgo = analysis4.getOptimizationAlgorithm()
optimizationAlgo.setMaximumCallsNumber(50)
optimizationAlgo.setMaximumAbsoluteError(1e-6)
analysis4.setOptimizationAlgorithm(optimizationAlgo)
analysis4.run()
myStudy.add(analysis4)
print("analysis=", analysis4)
print("isBayesian=", analysis4.getResult().getCalibrationResult().isBayesian())


# analysis check

# - check two outputs
stressObs.stack(stressObs * 2)
stressObs.setDescription(["sigma", "fakeSigma"])

observations2 = persalys.Observations("obs2", model, strainObs, stressObs)

analysis5 = persalys.CalibrationAnalysis("myAnalysis5", observations2)
analysis5.run()

fakeIn = persalys.Input("fakeIn")
model.addInput(fakeIn)

# - check fixed input order
analysis6 = persalys.CalibrationAnalysis("myAnalysis6", observations)
analysis6.setCalibratedInputs(
    ["R", "C"], ot.Dirac([700e6, 2500e6]), ["fakeIn", "gam"], [0.0, 7.0]
)
analysis6.run()

# - check calibrated input order
analysis7 = persalys.CalibrationAnalysis("myAnalysis7", observations)
analysis7.setCalibratedInputs(
    ["C", "R"], ot.Dirac([2500e6, 700e6]), ["gam", "fakeIn"], [7.0, 0.0]
)
analysis7.run()

# - check observed input order
strainObs2 = ot.Normal().getSample(nbObs)
strainObs2.stack(strainObs)
strainObs2.setDescription(["fakeIn", "strain"])

observations3 = persalys.Observations("obs3", model, strainObs2, stressObs)

analysis8 = persalys.CalibrationAnalysis("myAnalysis8", observations3)
analysis8.setCalibratedInputs(["R", "C"], ot.Dirac([700e6, 2500e6]), ["gam"], [7.0])
analysis8.run()

# - check exceptions
try:
    analysis6.setCalibratedInputs(
        ["R", "C"], ot.Dirac([700e6, 2400e6]), ["gam", "fakeIn"], [7.0, 0.0]
    )
except Exception as e:
    print(
        "InvalidArgumentException occurred: %s" % ("InvalidArgumentException" in str(e))
    )
try:
    analysis6.setCalibratedInputs(
        ["R", "C", "fakeIn"], ot.Dirac([700e6, 2400e6, 4.0]), ["gam"], [7.0]
    )
except Exception as e:
    print(
        "InvalidArgumentException occurred: %s" % ("InvalidArgumentException" in str(e))
    )
try:
    analysis6.setCalibratedInputs(
        ["R", "C"], ot.Dirac([700e6, 2400e6, 4.0]), ["gam", "C"], [7.0, 1e9]
    )
except Exception as e:
    print(
        "InvalidArgumentException occurred: %s" % ("InvalidArgumentException" in str(e))
    )
try:
    analysis6.setCalibratedInputs(["R", "C"], ot.Normal(3), ["gam"], [7.0])
except Exception as e:
    print(
        "InvalidArgumentException occurred: %s" % ("InvalidArgumentException" in str(e))
    )


print("\nthetaMAP=", analysis.getResult().getCalibrationResult().getParameterMAP())
print("thetaMAP CI=\n", analysis.getResult().getConfidenceInterval())

# print("\nthetaMAP=", analysis2.getResult().getCalibrationResult().getParameterMAP())
# print("thetaMAP CI=\n", analysis2.getResult().getConfidenceInterval())
thetaMAP = analysis2.getResult().getCalibrationResult().getParameterMAP()
ott.assert_almost_equal(thetaMAP, [7.73108e08, 3.55285e09], 1e-2, 1)


print("\nthetaMAP=", analysis3.getResult().getCalibrationResult().getParameterMAP())
print("thetaMAP CI=\n", analysis3.getResult().getConfidenceInterval())

# print("\nthetaMAP=", analysis4.getResult().getCalibrationResult().getParameterMAP())
# print("thetaMAP CI=\n", analysis4.getResult().getConfidenceInterval())
thetaMAP = analysis4.getResult().getCalibrationResult().getParameterMAP()
ott.assert_almost_equal(thetaMAP, [7.47517e08, 2.83374e09, 9.58942], 1e-2, 1)

print("\nthetaMAP=", analysis5.getResult().getCalibrationResult().getParameterMAP())
print("thetaMAP CI=\n", analysis5.getResult().getConfidenceInterval())
print("\nthetaMAP=", analysis6.getResult().getCalibrationResult().getParameterMAP())
print("thetaMAP CI=\n", analysis6.getResult().getConfidenceInterval())
print("\nthetaMAP=", analysis7.getResult().getCalibrationResult().getParameterMAP())
print("thetaMAP CI=\n", analysis7.getResult().getConfidenceInterval())
print("\nthetaMAP=", analysis8.getResult().getCalibrationResult().getParameterMAP())
print("thetaMAP CI=\n", analysis8.getResult().getConfidenceInterval())


# script
script = myStudy.getPythonScript()
print(script)
exec(script)



#! /usr/bin/env python

import openturns as ot
import persalys

code = """
import openturns as ot
def _exec(x1, x2, x3, x4, x5, x6, x7, x8, x9, x10, x11, x12, x13, x14, x15, x16, x17, x18, x19, x20):
    ot.RandomGenerator.SetSeed(123)
    b0 = ot.DistFunc.rNormal()
    alpha = ot.DistFunc.rNormal(10)
    beta =  ot.DistFunc.rNormal(6*14)
    gamma =  ot.DistFunc.rNormal(20*14)

    # b1
    b1 = [20] * 10 + list(alpha)

    # b2
    b2 = [[0] *20] * 20
    for i in range(6):
        for j in range(6):
            b2[i][j] = -15.0

    # Take into account beta
    k = 0
    for i in range(6):
        for j in range(14):
            b2[i][j + 6] = beta[k]
            k = k + 1

    # Take into account gamma
    k = 0
    for i in range(6, 20):
        for j in range(20):
            b2[i][j] = gamma[k]

    # b3
    b3 = [[[0]*20]*20]*20
    for i in range(5):
        for j in range(5):
            for k in range(5):
                b3[i][j][k] = -10.0

    # b4
    b4 = [[[[0]*20]*20]*20]*20
    for i in range(4):
        for j in range(4):
            for k in range(4):
                for l in range(4):
                    b4[i][j][k][l] = 5

    # X is a list, transform it into array
    X = ot.Point([x1, x2, x3, x4, x5, x6, x7, x8, x9, x10, x11, x12, x13, x14, x15, x16, x17, x18, x19, x20])
    w = (X - [0.5]*20)*2
    for k in [2,4,6]:
        w[k] = 2.0 * (1.1 * X[k] / (X[k] + 0.1) - 0.5)

    y = w.dot(b1)

    # Morris function
    for i in range(19):
        for j in range(i + 1, 20):
            y +=  b2[i][j] * w[i] * w[j]
    for i in range(18):
        for j in range(i + 1, 19):
            for k in range(j + 1, 20):
                y += b3[i][j][k] * w[i] * w[j] * w[k]

    for i in range(17):
        for j in range(i + 1, 18):
            for k in range(j + 1, 20):
                for l in range(k + 1, 20):
                    y += b4[i][j][k][l] * w[i] * w[j] * w[k] * w[l]

    y = y + b0
    y_fake = 0.

    return y

"""


myStudy = persalys.Study("myStudy")

inputs = []
for i in range(20):
    inputs.append(persalys.Input("x" + str(i + 1), ot.Uniform(0, 1)))
y = persalys.Output("y")
y_fake = persalys.Output("y_fake")
print(inputs)
model = persalys.PythonPhysicalModel("MorrisModel", inputs, [y, y_fake], code)
model.setProcessNumber(1)
myStudy.add(model)

# Morris ##
analysis = persalys.MorrisAnalysis("aMorris", model)
analysis.setInterestVariables(["y"])
analysis.setLevel(4)
analysis.setTrajectoriesNumber(4)
analysis.setSeed(2)
myStudy.add(analysis)
print(analysis)

analysis.run()

print("error message=", analysis.getErrorMessage())
print("result=", analysis.getResult())
print("meanAbsEE =", analysis.getResult().getMeanAbsoluteElementaryEffects(0))
# script
script = myStudy.getPythonScript()

persalys.Study.Add(myStudy)
print(script)
exec(script)



#!/usr/bin/env python

import openturns as ot
import persalys

# mesh model
meshModel0 = persalys.GridMeshModel()

print(meshModel0)

# mesh model
meshModel1 = persalys.GridMeshModel(
    [persalys.Variable("x", "aDescription")], ot.Interval(0.0, 12.0), ot.Indices(1, 20)
)

print(meshModel1)

filename = "mesh.csv"
mesh = ot.IntervalMesher(ot.Indices(1, 20)).build(ot.Interval(0.0, 12.0))
mesh.getVertices().exportToCSVFile(filename)

# mesh model
meshModel2 = persalys.ImportedMeshModel()

print(meshModel2)

# mesh model
meshModel3 = persalys.ImportedMeshModel(
    "mesh.csv", [persalys.Variable("z", "aDescription")], ot.Indices(1, 0)
)

print(meshModel3)



#! /usr/bin/env python

import openturns as ot
import persalys
import os

anOTStudy = persalys.Study("anOTStudy")

# Model
X0 = persalys.Input("X0", 1, ot.Normal())
X1 = persalys.Input("X1", 2, ot.Normal())
Y0 = persalys.Output("Y0")

model = persalys.SymbolicPhysicalModel("aModelPhys", [X0, X1], [Y0], ["sin(X0)+8*X1"])
anOTStudy.add(model)

# Design of Experiment ##
aDesign = persalys.GridDesignOfExperiment("aDesign_1", model, [[0.9, 1.1], [1.8, 2.2]])
anOTStudy.add(aDesign)

aDesign.run()
print("outs=", aDesign.getResult().getDesignOfExperiment().getOutputSample())

# Design of Experiment ##
filename = "normal.csv"
ot.Normal(3).getSample(10).exportToCSVFile(filename)
aDesign2 = persalys.ImportedDesignOfExperiment("aDesign_2", model, filename, [0, 2])
anOTStudy.add(aDesign2)

aDesign2.run()
print("outs=", aDesign2.getResult().getDesignOfExperiment().getOutputSample())

# Design of Experiment - imported - already evaluated##
aDesign2_1 = persalys.ImportedDesignOfExperiment(
    "aDesign_2_1", model, filename, [0, 2], [1]
)
anOTStudy.add(aDesign2_1)

# Considered already evaluated, do not run it
# aDesign2_1.run()
print("outs=", aDesign2_1.getResult().getDesignOfExperiment().getOutputSample())

# Design of Experiment ##
aDesign3 = persalys.ProbabilisticDesignOfExperiment(
    "aDesign_3", model, 10, "QUASI_MONTE_CARLO"
)
anOTStudy.add(aDesign3)

aDesign3.run()
print("outs=", aDesign3.getResult().getDesignOfExperiment().getOutputSample())

# Design of Experiment ##
aDesign4 = persalys.FixedDesignOfExperiment("aDesign_4", model)
inputSample = ot.LHSExperiment(model.getDistribution(), 10).generate()
# inputSample.stack(ot.Sample(10, [0.5, 1.3]))
aDesign4.setOriginalInputSample(inputSample)
anOTStudy.add(aDesign4)

aDesign4.run()

print("outs=", aDesign4.getResult().getDesignOfExperiment().getOutputSample())

# 3D Model to test Space filling algos
X0 = persalys.Input("X0", 1, ot.Normal())
X1 = persalys.Input("X1", 2, ot.Normal())
X2 = persalys.Input("X2", 3, ot.Normal())
Y0 = persalys.Output("Y0")

model = persalys.SymbolicPhysicalModel(
    "aModelPhys", [X0, X1, X2], [Y0], ["sin(X0)+8*X1*X2"]
)
anOTStudy2 = persalys.Study("anOTStudy2")
anOTStudy2.add(model)

# Design of Experiment SALHS-PhiP#
aDesign5 = persalys.ProbabilisticDesignOfExperiment(
    "aDesign_5", model, 10, "SALHS", "PhiP"
)
anOTStudy2.add(aDesign5)

aDesign5.run()
print("outs=", aDesign5.getResult().getDesignOfExperiment().getOutputSample())

# Design of Experiment SALHS-minDist#
aDesign6 = persalys.ProbabilisticDesignOfExperiment(
    "aDesign_6", model, 10, "SALHS", "minDist"
)
aDesign6.run()

print("outs=", aDesign6.getResult().getDesignOfExperiment().getOutputSample())

# Design of Experiment SALHS-C2#
aDesign7 = persalys.ProbabilisticDesignOfExperiment(
    "aDesign_7", model, 10, "SALHS", "C2", 1000
)
anOTStudy2.add(aDesign7)

aDesign7.run()
print("outs=", aDesign7.getResult().getDesignOfExperiment().getOutputSample())

# Design of Experiment MCLHS-PhiP#
aDesign8 = persalys.ProbabilisticDesignOfExperiment(
    "aDesign_8", model, 10, "MCLHS", "PhiP", 1000
)
anOTStudy2.add(aDesign8)

aDesign8.run()
print("outs=", aDesign8.getResult().getDesignOfExperiment().getOutputSample())

# Design of Experiment MCLHS-minDist#
aDesign9 = persalys.ProbabilisticDesignOfExperiment(
    "aDesign_9", model, 10, "MCLHS", "minDist", 1000
)
anOTStudy2.add(aDesign9)

aDesign9.run()
print("outs=", aDesign9.getResult().getDesignOfExperiment().getOutputSample())

# Design of Experiment MCLHS-C2#
aDesign10 = persalys.ProbabilisticDesignOfExperiment(
    "aDesign_10", model, 10, "MCLHS", "C2", 1000
)
anOTStudy2.add(aDesign10)

aDesign10.run()
print("outs=", aDesign10.getResult().getDesignOfExperiment().getOutputSample())

# Design of Experiment with failed points ##
code = "import time\ndef _exec(X0,X1):\n    Y0 = 1/(X0+X1)\n    return Y0\n"
model2 = persalys.PythonPhysicalModel("aModelPhys2", [X0, X1], [Y0], code)
model2.setProcessNumber(1)
anOTStudy.add(model2)

aDesign11 = persalys.GridDesignOfExperiment(
    "aDesign_11", model2, [[-3, 0, 3], [-3, 0, 3]]
)
anOTStudy.add(aDesign11)

aDesign11.run()
for err_msg in aDesign11.getErrorDescription():
    assert "ZeroDivisionError" in err_msg

model2.setParallel(False)
aDesign12 = persalys.GridDesignOfExperiment(
    "aDesign_12", model2, [[-3, 0, 3], [-3, 0, 3]]
)
anOTStudy.add(aDesign12)

aDesign12.run()
for err_msg in aDesign11.getErrorDescription():
    assert "ZeroDivisionError" in err_msg

# script
script = anOTStudy.getPythonScript()
print(script)
exec(script)

script = anOTStudy2.getPythonScript()
print(script)
exec(script)


os.remove(filename)



#!/usr/bin/env python

import openturns as ot
import openturns.testing as ott
import persalys
import os
import platform

myStudy = persalys.Study("myStudy")

key_platform = (platform.system(), platform.architecture()[0])
# Call to either 'platform.system' or 'platform.architecture' *after*
# importing pyfmi causes a segfault.
dict_platform = {("Linux", "64bit"): "linux64", ("Windows", "64bit"): "win64"}

path_here = os.path.dirname(os.path.abspath(__file__))
try:
    directory_platform = dict_platform[key_platform]
except KeyError:
    raise RuntimeError("Not available")

# CS model
path_fmu = os.path.join(path_here, "fmu", directory_platform, "deviation.fmu")

E = persalys.Input("E", 1.0, ot.Normal(), "E description")
F = persalys.Input("F", 1.0)
II = persalys.Input("I", 1.0)
L = persalys.Input("L", 1.0)
y = persalys.Output("y", "deviation")

inputs = [E, F, II, L]
outputs = [y]
model = persalys.FMIPhysicalModel("myPhysicalModel", inputs, outputs, path_fmu)
model.setParallel(True)
myStudy.add(model)

E_d = ot.Beta(0.93, 3.2 - 0.93, 28000000.0, 48000000.0)
F_d = ot.LogNormalMuSigma(30000.0, 9000.0, 15000.0).getDistribution()
L_d = ot.Uniform(250.0, 260.0)
I_d = ot.Beta(2.5, 4.0 - 2.5, 310.0, 450.0)
distribution = ot.JointDistribution([E_d, F_d, I_d, L_d])
x = [distribution.getMean(), distribution.computeQuantile(0.5)]

f = model.getFunction()
y = f(x)
ott.assert_almost_equal(y, [[12.3369], [12.8965]])

# ME model

path_fmu = os.path.join(path_here, "fmu", directory_platform, "epid.fmu")

healing_rate = persalys.Input("healing_rate", 0.02)
infection_rate = persalys.Input("infection_rate", 0.007)
infected = persalys.Output("infected")

inputs = [healing_rate, infection_rate]
outputs = [infected]
model2 = persalys.FMIPhysicalModel("myPhysicalModel2", inputs, outputs, path_fmu)
model2.setParallel(True)
myStudy.add(model2)
x = [[0.01, 0.004],
     [0.03, 0.014]]

f = model2.getFunction()
y = f(x)
ott.assert_almost_equal(y, [[434.668], [159.367]])

# script
script = myStudy.getPythonScript()
print("script=", script)
exec(script)



#!/usr/bin/env python

import openturns as ot
import openturns.testing
import persalys

myStudy = persalys.Study("myStudy")

# Model
dist_Q = ot.Gumbel(1.0 / 558.0, 1013.0)
dist_Ks = ot.TruncatedDistribution(
    ot.Normal(30.0, 7.5), 0, ot.TruncatedDistribution.LOWER
)
dist_Zv = ot.Uniform(49.0, 51.0)
dist_Zm = ot.Uniform(54.0, 56.0)

Q = persalys.Input("Q", 1000.0, dist_Q, "Debit maximal annuel (m3/s)")
Ks = persalys.Input("Ks", 30.0, dist_Ks, "Strickler (m^(1/3)/s)")
Zv = persalys.Input("Zv", 50.0, dist_Zv, "Cote de la riviere en aval (m)")
Zm = persalys.Input("Zm", 55.0, dist_Zm, "Cote de la riviere en amont (m)")
S = persalys.Output("S", "Surverse (m)")

model = persalys.SymbolicPhysicalModel(
    "myPhysicalModel",
    [Q, Ks, Zv, Zm],
    [S],
    ["(Q/(Ks*300.*sqrt((Zm-Zv)/5000)))^(3.0/5.0)+Zv-55.5-3."],
)
myStudy.add(model)

# limit state ##
limitState = persalys.LimitState("limitState1", model, "S", ot.Greater(), -1.0)
myStudy.add(limitState)

# Monte Carlo ##
montecarlo = persalys.MonteCarloReliabilityAnalysis("myMonteCarlo", limitState)
montecarlo.setMaximumCalls(10000)
montecarlo.setMaximumCoefficientOfVariation(0.01)
montecarlo.setBlockSize(1)
myStudy.add(montecarlo)

montecarlo.run()
montecarloResult = montecarlo.getResult()

# Comparaison
openturns.testing.assert_almost_equal(
    montecarloResult.getSimulationResult().getProbabilityEstimate(),
    9.999999999999946e-05,
    1e-6,
)

# FORM-IS ##
formIS = persalys.FORMImportanceSamplingAnalysis("myformIS", limitState)
formIS.setMaximumCoefficientOfVariation(0.01)
formIS.setMaximumCalls(10000)
formIS.setBlockSize(1000)
myStudy.add(formIS)

formIS.run()
formISResult = formIS.getResult()

# Comparaison
openturns.testing.assert_almost_equal(
    formISResult.getSimulationResult().getProbabilityEstimate(), 0.00022, 1e-5, 1e-5
)

# script
script = myStudy.getPythonScript()
print(script)
exec(script)



#! /usr/bin/env python

import openturns as ot
import persalys

myStudy = persalys.Study("myStudy")

# Model
x0 = persalys.Input("x0", ot.Normal())
x1 = persalys.Input("x1", ot.Normal())
y00 = persalys.Output("fake_y0")
y00.setIsSelected(False)
y0 = persalys.Output("y0")

formula_y00 = "x0"
formula_y0 = "10+3*x0+x1"
model = persalys.SymbolicPhysicalModel(
    "aModel", [x0, x1], [y00, y0], [formula_y00, formula_y0]
)
myStudy.add(model)

# SRC ##
analysis = persalys.SRCAnalysis("aSRC", model)
analysis.setSimulationsNumber(1000)
analysis.setSeed(2)
myStudy.add(analysis)
print(analysis)

analysis.run()

print("result=", analysis.getResult())

# SRC ##
X2 = persalys.Input("x2", 10)
model.addInput(X2)
model.addOutput(persalys.Output("y1"))
model.setFormula("y1", "3*x0 + x1 + x2")

analysis2 = persalys.SRCAnalysis("aSRC2", model)
analysis2.setSimulationsNumber(1000)
analysis2.setSeed(2)
analysis2.setInterestVariables(["y1"])
myStudy.add(analysis2)
print(analysis2)

analysis2.run()

result2 = analysis2.getResult()
print("result=", result2)

# script
script = myStudy.getPythonScript()
print(script)
exec(script)

# check getImplementation
analysis3 = persalys.Analysis(analysis2).getImplementation()
assert analysis3.getSimulationsNumber() == 1000



#! /usr/bin/env python

import openturns as ot
import openturns.testing as ott
import persalys


pyscript = "def _exec(x, y, p):\n    w, z = 3*x+4*y+p,x+y+p\n    return w, z\n"

model = persalys.YACSPhysicalModel("myPhysicalModel", [], [], pyscript)
model.setUseYACS(True)

X = [[1, 2, 4], [2, 3, 5], [6, 1, 3]]

Y_ref = [[15, 7], [23, 10], [25, 10]]

# full eval
f = model.getFunction()
Y = f(X)
print(Y)
ott.assert_almost_equal(Y, Y_ref)

# partial eval
f = model.getFunction(["w"])
Y = f(X)
print(Y)
ott.assert_almost_equal(Y, ot.Sample(Y_ref).getMarginal(0))

# script
myStudy = persalys.Study("myStudy")
myStudy.add(model)
script = myStudy.getPythonScript()
# print('script=', script)
exec(script)

# test for output variables order
pyscript = "def _exec(inp1):\n    aaa,bbb,ddd,ccc = 1.,2.,3.,inp1\n    return aaa,bbb,ddd,ccc\n"
model = persalys.YACSPhysicalModel("order", [], [], pyscript)
model.setUseYACS(True)
x = [8.0]
y_ref = [1.0, 2.0, 3.0, 8.0]
f = model.getFunction()
y = f(x)
# ott.assert_almost_equal(y, y_ref)

pyscript = "def _exec(x1, x2):\n    y1, y2 = 1/x1, x2\n    return y1, y2\n"
model = persalys.YACSPhysicalModel("myPhysicalModelError", [], [], pyscript)
model.setUseYACS(True)
model.setParallel(True)
model.setProcessNumber(4)

X = [[0, 1], [1, 1]]

design = persalys.FixedDesignOfExperiment("myDesign", model, X)
design.run()
print("outs=", design.getErrorDescription())

Y_ref = [[1, 1]]
ott.assert_almost_equal(
    design.getResult().getDesignOfExperiment().getOutputSample(), Y_ref
)



#!/usr/bin/env python

import openturns as ot
import persalys

Study_0 = persalys.Study("Study_0")
persalys.Study.Add(Study_0)

# variables
z0 = persalys.Input("z0", 100)
v0 = persalys.Input("v0", 55)
m = persalys.Input("m", 80)
c = persalys.Input("c", 16)
z1 = persalys.Output("z1", "")
z2 = persalys.Output("z2", "fake output")
inputs = [z0, v0, m, c]
outputs = [z1, z2]

# mesh model
meshModel = persalys.GridMeshModel(ot.Interval(0.0, 12.0), [20])

# Python model
code = "from math import exp\n\ndef _exec(z0,v0,m,c):\n    g = 9.81\n    zmin = 0.\n    tau = m / c\n    vinf = -m * g / c\n\n"
code += "    # mesh nodes\n    t = getMesh().getVertices()\n\n"
code += "    z = [max(z0 + vinf * t_i[0] + tau * (v0 - vinf) * (1 - exp(-t_i[0] / tau)), zmin) for t_i in t]\n"
code += "    z2 = [2*max(z0 + vinf * t_i[0] + tau * (v0 - vinf) * (1 - exp(-t_i[0] / tau)), zmin) for t_i in t]\n\n"
code += "    return z, z2"
PhysicalModel_1 = persalys.PythonFieldModel(
    "PhysicalModel_1", meshModel, inputs, outputs, code
)
Study_0.add(PhysicalModel_1)

# evaluation
analysis1 = persalys.FieldModelEvaluation("analysis1", PhysicalModel_1)
Study_0.add(analysis1)
print(analysis1)

analysis1.run()
result1 = analysis1.getProcessSample()
print("result=", result1)

analysis2 = persalys.FieldModelEvaluation(
    "analysis2", PhysicalModel_1, [100, 55, 81, 17]
)
analysis2.setInterestVariables(["z2"])
Study_0.add(analysis2)
print(analysis2)

analysis2.run()
result2 = analysis2.getProcessSample()
print("result=", result2)

# script
script = Study_0.getPythonScript()
print(script)
exec(script)



#!/usr/bin/env python

import openturns as ot
import openturns.testing
import persalys

ot.RandomGenerator.SetSeed(0)

myStudy = persalys.Study('myStudy')
persalys.Study.Add(myStudy)

# Models
x1 = persalys.Input('x1', 0.2, ot.Uniform(0., 10.))
x2 = persalys.Input('x2', 1.2, ot.Uniform(0., 10.))
x3 = persalys.Input('x3', 1.0)
fake_var = persalys.Output('fake_var')
fake_var.setIsSelected(False)
y0 = persalys.Output('y0')
fake_y0 = persalys.Output('fake_y0')
y1 = persalys.Output('y1')

# model 1 ##
formula_fake_var = 'x1+'
formula_y0 = 'cos(0.5*x1) + sin(x2)'
formula_y1 = 'cos(0.5*x1) + sin(x2) + x3'
model1 = persalys.SymbolicPhysicalModel('model1', [x1, x2, x3], [fake_var, y0, fake_y0, y1], [
                                         formula_fake_var, formula_y0, formula_y0, formula_y1])

myStudy.add(model1)

# model 3 ##
filename = 'data.csv'
cDist = ot.JointDistribution(
    [ot.Normal(), ot.Gumbel(), ot.Normal(), ot.Uniform()],
                                ot.BlockIndependentCopula([ot.IndependentCopula(2), ot.GumbelCopula()]))
sample = cDist.getSample(20)
sample.exportToCSVFile(filename, ' ')
model3 = persalys.DataModel(
    'model3', 'data.csv', [0, 2, 3], [1], ['x_0', 'x_2', 'x_3'], ['x_1'])
myStudy.add(model3)

# Design of Experiment ##

probaDesign = persalys.ProbabilisticDesignOfExperiment('probaDesign', model1, 20, "MONTE_CARLO")
probaDesign.run()
myStudy.add(probaDesign)

# 1- meta model1 ##

# 1-a Kriging ##
kriging = persalys.KrigingAnalysis('kriging', probaDesign)
kriging.setBasis(ot.LinearBasisFactory(2).build())
kriging.setCovarianceModel(ot.MaternModel(2))
kriging.setTestSampleValidation(True)
kriging.setKFoldValidation(True)
kriging.setInterestVariables(['y0', 'y1'])
myStudy.add(kriging)

# 1-b Chaos ##
chaos1 = persalys.FunctionalChaosAnalysis('chaos_1', probaDesign)
chaos1.setChaosDegree(7)
chaos1.setSparseChaos(True)
chaos1.setTestSampleValidation(True)
chaos1.setKFoldValidation(True)
chaos1.setInterestVariables(['y1'])
myStudy.add(chaos1)

# 2- central tendancy ##

# 2-a Monte Carlo ##
monteCarlo = persalys.MonteCarloAnalysis('MonteCarlo', model1)
monteCarlo.setIsConfidenceIntervalRequired(False)
monteCarlo.setMaximumCoefficientOfVariation(-1.)
monteCarlo.setMaximumElapsedTime(1000)
monteCarlo.setMaximumCalls(20)
monteCarlo.setSeed(2)
monteCarlo.setInterestVariables(['y0', 'y1'])
myStudy.add(monteCarlo)

# 2-b Taylor Expansion ##
taylor = persalys.TaylorExpansionMomentsAnalysis('Taylor', model1)
taylor.setInterestVariables(['y0', 'y1'])
myStudy.add(taylor)

# 2-c Taylor Expansion which generate an error
taylor2 = persalys.TaylorExpansionMomentsAnalysis('Taylor2', model1)
taylor2.setInterestVariables(['fake_var'])
myStudy.add(taylor2)

# 3- reliability ##

# limit state ##
limitState = persalys.LimitState(
    'aLimitState', model1, 'y1', ot.Greater(), 0.5)
myStudy.add(limitState)

optimAlgo = ot.AbdoRackwitz()
optimAlgo.setMaximumIterationNumber(150)
optimAlgo.setMaximumAbsoluteError(1e-3)

# 3-a Monte Carlo ##
monteCarloReliability = persalys.MonteCarloReliabilityAnalysis('MonteCarloReliability', limitState)
monteCarloReliability.setMaximumCoefficientOfVariation(-1.)
monteCarloReliability.setMaximumElapsedTime(1000)
monteCarloReliability.setMaximumCalls(20)
monteCarloReliability.setSeed(2)
myStudy.add(monteCarloReliability)

# 3-b FORM IS ##
form_is = persalys.FORMImportanceSamplingAnalysis('FORM_IS', limitState)
form_is.setOptimizationAlgorithm(optimAlgo)
form_is.setMaximumCoefficientOfVariation(-1.)
form_is.setMaximumElapsedTime(1000)
form_is.setMaximumCalls(20)
form_is.setSeed(2)
myStudy.add(form_is)

# 3-c FORM ##
form = persalys.FORMAnalysis('FORM', limitState)
form.setOptimizationAlgorithm(optimAlgo)
myStudy.add(form)

# 3-d SORM ##
sorm = persalys.SORMAnalysis('SORM', limitState)
sorm.setOptimizationAlgorithm(optimAlgo)
myStudy.add(sorm)

# 4- sensitivity ##

# 4-a Sobol ##
sobol = persalys.SobolAnalysis('Sobol', model1)
sobol.setMaximumConfidenceIntervalLength(-1)
sobol.setMaximumElapsedTime(1000)
sobol.setMaximumCalls(20)
sobol.setReplicationSize(3)
sobol.setBlockSize(1)
sobol.setSeed(2)
sobol.setInterestVariables(['y0', 'y1'])
myStudy.add(sobol)

# 4-b SRC ##
src = persalys.SRCAnalysis('SRC', model1)
src.setSimulationsNumber(20)
src.setSeed(2)
src.setInterestVariables(['y0', 'y1'])
myStudy.add(src)

# 7- data analysis ##
dataAnalysis = persalys.DataAnalysis('DataAnalysis', model3)
myStudy.add(dataAnalysis)

# 8- Marginals inference ##
inference = persalys.InferenceAnalysis('inference', model3)
inference.setInterestVariables(['x_0', 'x_3'])
factories = [ot.NormalFactory(), ot.GumbelFactory()]
inference.setDistributionsFactories('x_3', factories)
inference.setLevel(0.1)
myStudy.add(inference)

# 9- Copula inference ##
copulaInference = persalys.CopulaInferenceAnalysis('copulaInference', model3)
factories = [ot.NormalCopulaFactory(), ot.GumbelCopulaFactory()]
copulaInference.setDistributionsFactories(['x_0', 'x_3'], factories)
myStudy.add(copulaInference)



#!/usr/bin/env python

import openturns as ot
import openturns.testing as ott
import persalys
import os
import math
import sys


# testcase A: parse output with tokens
with open("input_template0y.txt", "w") as f:
    f.write("X0=@X0\n")
    f.write("X1=@X1\n")
    f.write("X2=@X2\n")
with open("external_program0y.py", "w") as f:
    f.write("import sys\n")
    f.write('exec(open("input.txt").read())\n')
    f.write("Y0=X0+X1+X2\n")
    f.write("Y1=X0+X1*X2\n")
    f.write('with open("output.txt", "w") as f:\n')
    f.write('    f.write("Y0=%.17e\\n" % Y0)\n')
    f.write('    f.write("Y1=%.17e\\n" % Y1)\n')


input_file = persalys.CouplingInputFile("input_template0y.txt")
input_file.setConfiguredPath("input.txt")
input_file.setVariables(["X0", "X1", "X2"], ["@X0", "@X1", "@X2"], ["", "", ""])
resource_file = persalys.CouplingResourceFile("external_program0y.py")
output_file = persalys.CouplingOutputFile("output.txt")
output_file.setVariables(["Y0", "Y1"], ["Y0=", "Y1="], [0, 0], [0, 0], [0, 0])
step = persalys.CouplingStep(
    sys.executable + " external_program0y.py",
    [input_file],
    [resource_file],
    [output_file],
)
model = persalys.YACSCouplingPhysicalModel("A", [step])

# single evaluation
x = [1.0, 2.0, 3.0]
f = model.getFunction()
y = f(x)
print(y)
ott.assert_almost_equal(y, [6.0, 7.0])

# sampling test
ot_function = model.getFunction()
Y = ot.CompositeRandomVector(ot_function, ot.RandomVector(ot.Normal(3)))
y = Y.getSample(50)
mean, stddev = y.computeMean(), y.computeStandardDeviation()
print(mean, stddev)
ott.assert_almost_equal(mean, [0, 0], 0.3, 0.3)
ott.assert_almost_equal(stddev, [math.sqrt(3.0), 1.415], 0.3, 0.3)

# cleanup
os.remove("input_template0y.txt")
os.remove("external_program0y.py")

# script
myStudy = persalys.Study("myStudy")
myStudy.add(model)
script = myStudy.getPythonScript()
# print('script=', script)
exec(script)



import os
import platform
import persalys

# import openturns as ot
import openturns.testing as ott

# testcase using ansys connector
key_platform = platform.system()
dict_exe = {
    "Linux": ["dummyAnsys", "./dummyAnsys"],
    "Windows": ["dummyAnsys.exe", "dummyAnsys.exe"],
}

pathHere = os.path.dirname(os.path.abspath(__file__))
pathResources = os.path.join(pathHere, "ansysConnector")
modelFile = os.path.join(pathResources, "BEAM.wbpj")
ansysExec = dict_exe[key_platform][0]
templateFile = "input.txt.in"

step = persalys.CouplingStep()
model = persalys.CouplingPhysicalModel("A", [step])

parser = persalys.AnsysParser(modelFile)
parser.populateCouplingStep(model, 0, templateFile)

# Required for testing because of dummyAnsys
step = model.getSteps()[0]
resourceFiles = step.getResourceFiles()
resourceFiles.add(persalys.CouplingResourceFile(ansysExec))
step.setResourceFiles(resourceFiles)
command = step.getCommand()
step.setCommand(dict_exe[key_platform][1] + command)

model.setSteps([step])
x = [1.0, 2.0, 3.0, 4.0, 5.0, 6.0]
f = model.getFunction()
y = f(x)
print(y)
ott.assert_almost_equal(y, [0.10662])

os.remove(templateFile)



#!/usr/bin/env python

import openturns as ot
import openturns.testing
import persalys

myStudy = persalys.Study("myStudy")

# Model
dist_E = ot.Beta(0.93, 3.2, 2.8e7, 4.8e7)
dist_F = ot.LogNormalMuSigma(30000.0, 9000.0, 15000).getDistribution()
dist_L = ot.Uniform(250, 260)
dist_I = ot.Beta(2.5, 4.0, 3.1e2, 4.5e2)

E = persalys.Input("E", 3e7, dist_E, "Young's modulus (Pa)")
F = persalys.Input("F", 3e4, dist_F, "Force applied (N)")
L = persalys.Input("L", 250, dist_L, "Length (m4)")
II = persalys.Input("I", 400, dist_I, "Section modulus (m4)")
y = persalys.Output("y", "Vertical deviation (m)")

model = persalys.SymbolicPhysicalModel(
    "myPhysicalModel", [E, F, L, II], [y], ["F*L^3/(3*E*I)"]
)
myStudy.add(model)

f = model.getFunction()
print(f([3e7, 3e4, 250, 400]))

# Sobol ##
sobol = persalys.SobolAnalysis("mySobol", model)
sobol.setMaximumConfidenceIntervalLength(-1)
sobol.setMaximumCalls(1000)
sobol.setReplicationSize(166)
myStudy.add(sobol)
sobol.run()
sobolResult = sobol.getResult()

# SRC ##
src = persalys.SRCAnalysis("mySRC", model, 1000)
myStudy.add(src)
src.run()
srcResult = src.getResult()


# Create the Spearman correlation matrix of the input random vector
RS = ot.CorrelationMatrix(2)
RS[0, 1] = -0.2
# Evaluate the correlation matrix of the Normal copula from RS
R = ot.NormalCopula.GetCorrelationFromSpearmanCorrelation(RS)
# Create the Normal copula parametrized by R
copula = ot.NormalCopula(R)
model.setCopula(["L", "I"], copula)

# Design of Experiment - Parametric analysis ##
bounds = persalys.GridDesignOfExperiment.GetDefaultBounds(model)
values = [[bounds.getLowerBound()[i], bounds.getUpperBound()[i]] for i in range(4)]
aDesign = persalys.GridDesignOfExperiment("aDesign", model, values)
myStudy.add(aDesign)
aDesign.run()

# Taylor Expansion ##
taylorExpansionsMoments = persalys.TaylorExpansionMomentsAnalysis(
    "myTaylorExpansionMoments", model
)
myStudy.add(taylorExpansionsMoments)
taylorExpansionsMoments.run()
taylorExpansionsMomentsResult = taylorExpansionsMoments.getResult()

# Monte Carlo ##
montecarlo = persalys.MonteCarloAnalysis("myMonteCarlo", model)
montecarlo.setMaximumCalls(1000)
myStudy.add(montecarlo)
montecarlo.run()
montecarloResult = montecarlo.getResult()

meanCI = montecarloResult.getMeanConfidenceInterval()
stdCi = montecarloResult.getStdConfidenceInterval()

# limit state ##
limitState = persalys.LimitState("limitState1", model, "y", ot.Greater(), 30.0)
myStudy.add(limitState)

# Monte Carlo ##
montecarloSimu = persalys.MonteCarloReliabilityAnalysis("myMonteCarloSimu", limitState)
montecarloSimu.setMaximumCalls(10000)
montecarloSimu.setBlockSize(10)
myStudy.add(montecarloSimu)
montecarloSimu.run()
montecarloSimuResult = montecarloSimu.getResult()

# script
script = myStudy.getPythonScript()
print(script)
exec(script)



#! /usr/bin/env python

import openturns as ot
import openturns.testing
import persalys
import os

myStudy = persalys.Study("myStudy")

# Model 1
filename = "data.csv"
ot.RandomGenerator.SetSeed(0)
ot.Normal(3).getSample(10).exportToCSVFile(filename)
inColumns = [0, 2]

model = persalys.DataModel("myDataModel", filename, inColumns)
myStudy.add(model)
print(model)

print("inputNames=", model.getInputNames())
print("outputNames=", model.getOutputNames())

# Data analysis ##
analysis = persalys.DataAnalysis("aDataAnalysis", model)
myStudy.add(analysis)

analysis.run()

result = analysis.getResult()
print("result=", result)
print("PDF=", result.getPDF())
print("CDF=", result.getCDF())
print("SurvivalFunction=", result.getSurvivalFunction())
print("outliers=", result.getOutliers())

# Comparaison
openturns.testing.assert_almost_equal(0.2012538261144671, result.getMean()[0][0], 1e-13)
openturns.testing.assert_almost_equal(
    -0.14315074899830527, result.getMean()[1][0], 1e-13
)

# Model 2
outColumns = [1]

model2 = persalys.DataModel(
    "myDataModel2", filename, inColumns, outColumns, ["var1", "var2"], ["var3"]
)
myStudy.add(model2)
print(model2)

print("inputNames=", model2.getInputNames())
print("outputNames=", model2.getOutputNames())

print("min=", model2.getListXMin())
print("max=", model2.getListXMax())

# Model 3
model3 = persalys.DataModel("myDataModel3", filename, inColumns, outColumns)
myStudy.add(model3)
print(model3)

print("inputNames=", model3.getInputNames())
print("outputNames=", model3.getOutputNames())

print("inputSample=", model3.getInputSample())
print("outputSample=", model3.getOutputSample())

print("min=", model3.getListXMin())
print("max=", model3.getListXMax())

# Quantile analysis
# Model 4
filename = "data_500.csv"
ot.RandomGenerator.SetSeed(0)
ot.Normal(3).getSample(500).exportToCSVFile(filename)
inColumns = [0, 1, 2]

model4 = persalys.DataModel("myDataModel4", filename, inColumns)
myStudy.add(model4)
print(model4)
print("inputNames=", model4.getInputNames())
print("outputNames=", model4.getOutputNames())

# Monte Carlo
analysis2 = persalys.QuantileAnalysis("aQuantileAnalysis", model4)
analysis2.setTargetProbabilities([[1e-2, 1e-3], [6e-3], [1e-1, 1e-2, 1e-3]])
analysis2.setTailTypes(
    [persalys.QuantileAnalysisResult.Upper | persalys.QuantileAnalysisResult.Lower | persalys.QuantileAnalysisResult.Bilateral,
     persalys.QuantileAnalysisResult.Lower,
     persalys.QuantileAnalysisResult.Bilateral])

analysis2.setType(persalys.QuantileAnalysisResult.MonteCarlo)
myStudy.add(analysis2)
analysis2.run()
result = analysis2.getResult()

x0ref = [[[-2.46389, -2.23369, -2.0035],
          [-2.51804, -2.4067, -2.29536]],
         [[1.92083, 2.30459, 2.68835],
          [2.85639, 3.15958, 3.46277]],
         [[-2.47455, -2.29006, -2.10558],
          [-2.48545, -2.4067, -2.32795]],
         [[2.41487, 3.02799, 3.64111],
          [2.94514, 3.15958, 3.37402]]]

x1ref = [[[-3.36709, -2.84159, -2.31609]]]

x2ref = [[[-1.73368, -1.5705, -1.40732],
          [-3.6264, -2.98535, -2.3443],
          [-3.32293, -3.09834, -2.87375]],
         [[1.51902, 1.69843, 1.87784],
          [2.29461, 2.5038, 2.71299],
          [2.53902, 2.63821, 2.7374]]]

for i, qx in enumerate(result.getQuantiles('X0')):
    for j, qxi in enumerate(qx):
        openturns.testing.assert_almost_equal(qxi, x0ref[i][j])
for i, qx in enumerate(result.getQuantiles('X1')):
    for j, qxi in enumerate(qx):
        openturns.testing.assert_almost_equal(qxi, x1ref[i][j])
for i, qx in enumerate(result.getQuantiles('X2')):
    for j, qxi in enumerate(qx):
        openturns.testing.assert_almost_equal(qxi, x2ref[i][j])

print(result.getSampleSizeValidity('X0', persalys.QuantileAnalysisResult.Upper))
print(result.getSampleSizeValidity('X0', persalys.QuantileAnalysisResult.Lower))
print(result.getSampleSizeValidity('X0', persalys.QuantileAnalysisResult.Bilateral))
print(result.getSampleSizeValidity('X1', persalys.QuantileAnalysisResult.Lower))
print(result.getSampleSizeValidity('X2', persalys.QuantileAnalysisResult.Bilateral))

# Generalized Pareto
# test wrong threshold
analysis2.setType(persalys.QuantileAnalysisResult.GeneralizedPareto)
try:
    analysis2.run()
except Exception as e:
    print(e)
analysis2.setTargetProbabilities([[1e-2, 1e-3], [6e-3], [2e-2, 1e-2, 1e-3]])
analysis2.run()

# correct threshold
ot.RandomGenerator.SetSeed(0)
analysis2.setThreshold(ot.Sample([[-1.2] * 3, [1.3] * 3]))
print(analysis2.getCDFThreshold())
analysis2.run()

lower = persalys.QuantileAnalysisResult.Lower
upper = persalys.QuantileAnalysisResult.Upper
result = analysis2.getResult()
x0ref = [[[-2.44824, -2.00098, -1.77584],
          [-3.41554, -2.30532, -1.81387]],
         [[1.963, 2.39031, 2.80089],
          [2.00437, 3.32964, 4.43636]],
         [[-2.792, -2.12135, -1.85867],
          [-3.78112, -2.3562, -1.89605]],
         [[2.1527, 2.70119, 3.29286],
          [2.21385, 3.56467, 5.04221]]]

x1ref = [[[-3.41533, -2.61337, -2.14595]]]

x2ref = [[[-2.69465, -2.37834, -1.91663],
          [-3.03319, -2.68309, -1.95693],
          [-4.48605, -3.59798, -1.98668]],
         [[1.85113, 2.16798, 2.66877],
          [1.88352, 2.30282, 2.93784],
          [1.9117, 2.56313, 3.89188]]]

for i, qx in enumerate(result.getQuantiles('X0')):
    for j, qxi in enumerate(qx):
        openturns.testing.assert_almost_equal(qxi, x0ref[i][j])
for i, qx in enumerate(result.getQuantiles('X1')):
    for j, qxi in enumerate(qx):
        openturns.testing.assert_almost_equal(qxi, x1ref[i][j])
for i, qx in enumerate(result.getQuantiles('X2')):
    for j, qxi in enumerate(qx):
        openturns.testing.assert_almost_equal(qxi, x2ref[i][j])

openturns.testing.assert_almost_equal(result.getPValue('X0', upper), 0.9378335994, 1e-3)
openturns.testing.assert_almost_equal(result.getPValue('X0', lower), 0.5392217742, 1e-3)
openturns.testing.assert_almost_equal(result.getPValue('X1', lower), 0.9037923523, 1e-3)
openturns.testing.assert_almost_equal(result.getPValue('X2', upper), 0.0674410269, 1e-3)
openturns.testing.assert_almost_equal(result.getPValue('X2', lower), 0.4360724055, 1e-3)

# test interest variables
analysis2.setInterestVariables(["X0", "X2"])
analysis2.setTailTypes(
    [persalys.QuantileAnalysisResult.Upper | persalys.QuantileAnalysisResult.Lower | persalys.QuantileAnalysisResult.Bilateral,
     persalys.QuantileAnalysisResult.Bilateral])
analysis2.setTargetProbabilities([[1e-2, 1e-3], [2e-2, 1e-2, 1e-3]])
analysis2.setThreshold(ot.Sample([[-1.2] * 2, [1.3] * 2]))
print(analysis2.getCDFThreshold())
analysis2.run()

# script
script = myStudy.getPythonScript()
print(script)
exec(script)

# check ambiguous import
sample = ot.Normal(2).getSample(10)
sample[0] = [1, 2]
inColumns = [0, 1]

for col_sep in [";", ",", " "]:
    for num_sep in [".", ","]:
        if col_sep == num_sep:
            continue
        with open(filename, "w") as csv:
            csv.write('"x"' + col_sep + '"y"\n')
            for p in sample:
                for j in range(len(p)):
                    csv.write(str(p[j]).replace(".", num_sep))
                    if j < len(p) - 1:
                        csv.write(col_sep)
                csv.write("\n")
        model = persalys.DataModel("myDataModel2", filename, inColumns)
        assert model.getSampleFromFile().getDimension() == 2, (
            "wrong dimension sep=" + col_sep
        )
        assert model.getSampleFromFile().getSize() == 10, "wrong size"
os.remove(filename)



#!/usr/bin/env python

import openturns as ot
import openturns.testing
import persalys

ot.RandomGenerator.SetSeed(0)

myStudy = persalys.Study('myStudy')
persalys.Study.Add(myStudy)

# Models
x1 = persalys.Input('x1', 0.2)
x2 = persalys.Input('x2', 1.2)
x3 = persalys.Input('x3', 1.0)
fake_var = persalys.Output('fake_var')
fake_var.setIsSelected(False)
y0 = persalys.Output('y0')
fake_y0 = persalys.Output('fake_y0')
y1 = persalys.Output('y1')

# symboilc model ##
formula_fake_var = 'x1'
formula_y0 = 'cos(0.5*x1) + sin(x2)'
formula_y1 = 'cos(0.5*x1) + sin(x2) + x3'
symbolicModel = persalys.SymbolicPhysicalModel('symbolicModel', [x1, x2, x3], [fake_var, y0, fake_y0, y1], [
                                         formula_fake_var, formula_y0, formula_y0, formula_y1])

myStudy.add(symbolicModel)

# python model ##
code = 'from math import cos, sin, sqrt\n\ndef _exec(x1, x2, x3):\n    y0 = cos(0.5*x1) + sin(x2) + sqrt(x3)\n    return y0\n'
pythonModel = persalys.PythonPhysicalModel('pythonModel', [x1, x2, x3], [y0], code)
myStudy.add(pythonModel)

filename = 'data.csv'
cDist = ot.JointDistribution([ot.Normal(), ot.Gumbel(), ot.Normal(), ot.Uniform()],
                                ot.BlockIndependentCopula([ot.IndependentCopula(2), ot.GumbelCopula()]))
sample = cDist.getSample(200)
sample.exportToCSVFile(filename, ' ')

# Designs of Experiment ##

# fixed design ##
ot.RandomGenerator.SetSeed(0)
fixedDesign = persalys.FixedDesignOfExperiment('fixedDesign', symbolicModel)
inputSample = ot.LHSExperiment(ot.JointDistribution([ot.Uniform(0., 10.), ot.Uniform(0., 10.)]), 10).generate()
inputSample.stack(ot.Sample(10, [0.5]))
fixedDesign.setOriginalInputSample(inputSample)
fixedDesign.run()
myStudy.add(fixedDesign)

# grid ##
values = [[0.5+i*1.5 for i in range(7)], [0.5+i*1.5 for i in range(7)], [1]]
grid = persalys.GridDesignOfExperiment('grid', symbolicModel, values)
myStudy.add(grid)

# importDesign ##
importDesign = persalys.ImportedDesignOfExperiment('importDesign', symbolicModel, 'data.csv', [0, 2, 3])
importDesign.run()
myStudy.add(importDesign)

# onePointDesign ##
onePointDesign = persalys.GridDesignOfExperiment('onePointDesign', pythonModel)
myStudy.add(onePointDesign)

# twoPointsDesign ##
twoPointsDesign = persalys.GridDesignOfExperiment('twoPointsDesign', pythonModel, [[0.2], [1.2], [-0.2, 1.]])
myStudy.add(twoPointsDesign)

# fixed DataModel ##
fixedDataModel = persalys.DataModel('fixedDataModel', fixedDesign.getOriginalInputSample(), fixedDesign.getResult().getDesignOfExperiment().getOutputSample())
myStudy.add(fixedDataModel)

# imported DataModel ##
importDataModel = persalys.DataModel('importDataModel', 'data.csv', [0, 2, 3], [1], ['x_0', 'x_2', 'x_3'], ['x_1'])
myStudy.add(importDataModel)

# model evaluation
evaluation1 = persalys.ModelEvaluation('evaluation1', symbolicModel)
myStudy.add(evaluation1)

# morris ##
try:
    morris = persalys.MorrisAnalysis('aMorris', symbolicModel)
    morris.setInterestVariables(['y0'])
    morris.setLevel(4)
    morris.setTrajectoriesNumber(10)
    morris.setSeed(2)
    myStudy.add(morris)
except:
    print("No Morris")

# optimization ##
optim = persalys.OptimizationAnalysis('optim', symbolicModel, 'TNC')
optim.setInterestVariables(['y1'])
optim.setVariableInputs(['x1', 'x2'])
optim.setMaximumCallsNumber(150)
optim.setMaximumAbsoluteError(1e-6)
optim.setMaximumRelativeError(1e-6)
optim.setMaximumResidualError(1e-6)
optim.setMaximumConstraintError(1e-6)
myStudy.add(optim)

# calibration ##

# a- observations
nbObs = 100
inObs = ot.Uniform(0., 10.).getSample(nbObs)
inObs.setDescription(['x1'])

y0Noise = ot.Normal(0, 0.1).getSample(nbObs)
y0Sample = ot.ParametricFunction(symbolicModel.getFunction('y0'), [1, 2], [1.2, 1.])(inObs)

y0Obs = y0Sample + y0Noise
y0Obs.setDescription(['y0'])
obs = persalys.Observations("observations", symbolicModel, inObs, y0Obs)
myStudy.add(obs)

# b- calibrationAnalysis
calibration = persalys.CalibrationAnalysis('calibration', obs)
calibration.setCalibratedInputs(['x2'], ot.Dirac([1.2]), ['x3'], [1.1])
calibration.setMethodName('GaussianNonlinear')
sigma = 0.15
errorCovariance = ot.CovarianceMatrix(1)
errorCovariance[0, 0] = sigma**2
calibration.setErrorCovariance(errorCovariance)
calibration.setBootStrapSize(25)
calibration.setConfidenceIntervalLength(0.99)

optimAlgo = calibration.getOptimizationAlgorithm()
optimAlgo.setMaximumCallsNumber(50)
optimAlgo.setMaximumAbsoluteError(1e-6)
optimAlgo.setMaximumRelativeError(1e-6)
optimAlgo.setMaximumResidualError(1e-6)
optimAlgo.setMaximumConstraintError(1e-6)
calibration.setOptimizationAlgorithm(optimAlgo)

myStudy.add(calibration)



#! /usr/bin/env python

import openturns as ot
import persalys

myStudy = persalys.Study("myStudy")

# Model
X0 = persalys.Input("X0", ot.Normal(1, 1))
X1 = persalys.Input("X1", ot.Normal(1, 1))
Y00 = persalys.Output("fake_Y0")
Y00.setIsSelected(False)
Y0 = persalys.Output("Y0")

formula_Y00 = "X0"
formula_Y0 = "sin(X0) + 8*X1"
model = persalys.SymbolicPhysicalModel(
    "aModelPhys", [X0, X1], [Y00, Y0], [formula_Y00, formula_Y0]
)
myStudy.add(model)

# Monte Carlo ##
analysis = persalys.MonteCarloAnalysis("myMonteCarlo", model)
analysis.setLevelConfidenceInterval(0.93)
analysis.setMaximumCalls(1000)
analysis.setMaximumCoefficientOfVariation(-1)
analysis.setMaximumElapsedTime(30)
analysis.setSeed(2)
myStudy.add(analysis)
print(analysis)

analysis.run()

result = analysis.getResult()
print("result=", result)
print("PDF=", result.getPDF())
print("CDF=", result.getCDF())
print("outliers=", result.getOutliers())

# Monte Carlo ##
analysis2 = persalys.MonteCarloAnalysis("myMonteCarlo2", model)
analysis2.setIsConfidenceIntervalRequired(False)
analysis2.setMaximumCoefficientOfVariation(0.02)
analysis2.setMaximumElapsedTime(100000)
analysis2.setBlockSize(100)
myStudy.add(analysis2)
print(analysis2)

analysis2.run()

result2 = analysis2.getResult()
print("result=", result2)
print("PDF=", result2.getPDF())
print("CDF=", result2.getCDF())
print("outliers=", result2.getOutliers())

# Monte Carlo ##
X2 = persalys.Input("X2", 2)
model.addInput(X2)
model.addOutput(persalys.Output("Y1"))
model.setFormula("Y1", "sin(X0) + 8*X1 + X2")

analysis3 = persalys.MonteCarloAnalysis("myMonteCarlo3", model)
analysis3.setMaximumCalls(1000)
analysis3.setInterestVariables(["Y1"])
myStudy.add(analysis3)
print(analysis3)

analysis3.run()

result3 = analysis3.getResult()
print("result=", result3)

# Monte Carlo - test max CI length stopping criterion ##

X2 = persalys.Input("X2", ot.Normal(0, 1))
Y2 = persalys.Output("Y2")
formula_Y2 = "X2*X2"

model2 = persalys.SymbolicPhysicalModel("aModelPhys2", [X2], [Y2], [formula_Y2])
myStudy.add(model2)

analysis4 = persalys.MonteCarloAnalysis("myMonteCarlo4", model2)
analysis4.setMaximumConfidenceIntervalLength(0.1)
analysis4.setInterestVariables(["Y2"])
myStudy.add(analysis4)
print(analysis4)

analysis4.run()

result4 = analysis4.getResult()
print("result=", result4)

# script
script = myStudy.getPythonScript()
print(script)
exec(script)



#!/usr/bin/env python

import openturns as ot
import openturns.testing as ott
import persalys

myStudy = persalys.Study("myStudy")

# Model
xi1 = persalys.Input("xi1", ot.Uniform(0.0, 10.0))
xi2 = persalys.Input("xi2", ot.Uniform(0.0, 10.0))
xi3 = persalys.Input("xi3", 0.5)
y00 = persalys.Output("fake_y0")
y00.setIsSelected(False)
y0 = persalys.Output("y0")
y1 = persalys.Output("y1")

formula_y00 = "xi1"
formula_y0 = "cos(0.5*xi1) + sin(xi2)"
formula_y1 = "cos(0.5*xi1) + sin(xi2) + xi3"
model = persalys.SymbolicPhysicalModel(
    "model", [xi1, xi2, xi3], [y00, y0, y1], [formula_y00, formula_y0, formula_y1]
)
myStudy.add(model)

# Design of Experiment ##
aDesign = persalys.FixedDesignOfExperiment("design", model)
inputSample = ot.LHSExperiment(model.getDistribution(), 50).generate()
inputSample.stack(ot.Sample(50, [0.5]))
aDesign.setOriginalInputSample(inputSample)
myStudy.add(aDesign)

aDesign.run()

# Chaos 1 ##
analysis = persalys.FunctionalChaosAnalysis("chaos_0", aDesign)
analysis.setChaosDegree(4)
analysis.setSparseChaos(True)
myStudy.add(analysis)
print(analysis)

analysis.run()

chaosResult = analysis.getResult()
sobolResult = chaosResult.getSobolResult()
# print("result=", chaosResult)
# print("functionalChaosResult", chaosResult.getFunctionalChaosResult())

# Comparaison
mean_ref = [-0.0114672, 0.488533]
variance_ref = [0.872994] * 2
fo_ref = [[0.52165, 0.47835]] * 2
to_ref = [[0.52165, 0.47835]] * 2

ott.assert_almost_equal(chaosResult.getMean(), mean_ref)
ott.assert_almost_equal(chaosResult.getVariance(), variance_ref)
ott.assert_almost_equal(sobolResult.getFirstOrderIndices(), fo_ref, 1e-3, 1e-3)
ott.assert_almost_equal(sobolResult.getTotalIndices(), to_ref, 1e-3, 1e-3)

# Chaos 2 ##
R = ot.CorrelationMatrix(2, [1, 0.5, 0.5, 1])
model.setCopula(["xi1", "xi2"], ot.NormalCopula(R))

design2 = persalys.ProbabilisticDesignOfExperiment("design2", model, 200, "MONTE_CARLO")
myStudy.add(design2)
design2.run()

analysis2 = persalys.FunctionalChaosAnalysis("chaos_1", design2)
analysis2.setChaosDegree(4)
analysis2.setAnalyticalValidation(True)
analysis2.setTestSampleValidation(True)
analysis2.setTestSampleValidationPercentageOfPoints(30)
analysis2.setTestSampleValidationSeed(2)
analysis2.setKFoldValidation(True)
analysis2.setKFoldValidationNumberOfFolds(4)
analysis2.setKFoldValidationSeed(2)
analysis2.setInterestVariables(["y1"])
myStudy.add(analysis2)
print(analysis2)

analysis2.run()

chaosResult2 = analysis2.getResult()
sobolResult2 = chaosResult2.getSobolResult()
print("result=", chaosResult2)
print("functionalChaosResult", chaosResult2.getFunctionalChaosResult())

ott.assert_almost_equal(chaosResult2.getAnalyticalValidation().getQ2(), [0.609145])
ott.assert_almost_equal(chaosResult2.getTestSampleValidation().getQ2(), [0.595006])
ott.assert_almost_equal(chaosResult2.getKFoldValidation().getQ2(), [0.601799])

# extract metamodel
metamodel = chaosResult2.getMetaModel()
ott.assert_almost_equal(metamodel.getFunction().getInputDimension(), 2)
ott.assert_almost_equal(metamodel.getFunction().getOutputDimension(), 1)
ott.assert_almost_equal(metamodel.getFunction()([0.5] * 2), [2.23523])
ott.assert_almost_equal(metamodel.getInputByName("xi1").getDistribution(), ot.Uniform(0., 10.))
assert model.getCopula() == ot.BlockIndependentCopula([ot.NormalCopula(ot.CorrelationMatrix(2, [1, 0.5, 0.5, 1]))])

# script
script = myStudy.getPythonScript()
print(script)
exec(script)



#!/usr/bin/env python

import openturns as ot
import openturns.testing
import persalys

myStudy = persalys.Study("myStudy")

# Model
R = persalys.Input(
    "R", 0.0, ot.LogNormalMuSigma(300.0, 30.0).getDistribution(), "Yield strength"
)
F = persalys.Input("F", 0.0, ot.Normal(75000.0, 5000.0), "Traction load")
G = persalys.Output("G", "deviation")
code = "from math import pi\n\ndef _exec(R, F):\n    G = R-F/(pi*100.0)\n    return G\n"

model = persalys.PythonPhysicalModel("myPhysicalModel", [R, F], [G], code)
myStudy.add(model)

f = model.getFunction()
print(f([300.0, 75000.0]))

# several outputs
model.setCode(
    "from math import pi\n\ndef _exec(R, F):\n    G =  6*F\n    H = 42.0\n    return G, H\n"
)
f = model.getFunction()
print(f([300.0, 75000.0]))

# test operator() (sample)
model.setCode(
    "from math import pi\n\ndef _exec(R, F):\n    G = 2*R-F/(pi*100.0)\n    return G\n"
)
f = model.getFunction()
print(f([[300.0, 75000.0]]))
print(f([[300.0, 75000.0], [400.0, 74000.0]]))

# test operator() (sample)
code = "from math import pi\nimport time\ndef _exec(R, F):\n"
code += (
    "    if R == 300.:\n      time.sleep(0.5)\n    G = 2*R-F/(pi*100.0)\n    return G\n"
)
model.setCode(code)
model.setProcessNumber(1)
plan_0 = persalys.FixedDesignOfExperiment(
    "plan_0", model, [[300.0, 75000.0], [400.0, 74000.0]]
)
plan_0.setBlockSize(2)
myStudy.add(plan_0)
plan_0.run()
print("Sample :\n %s" % plan_0.getResult().getDesignOfExperiment().getSample())

# model with an error
model.setCode(
    "from math import pi\n\ndef _exec(R, F):\n    G = 2*R-F/(pi*100.0)/0.\n    return G\n"
)
f = model.getFunction()
try:
    print(f([[300.0, 75000.0]]))
except Exception as e:
    ok = "ZeroDivisionError" in str(e)
    print(f"ZeroDivisionError occurred: {ok}")
    print("Error on the line 4 : %s" % ("line 4" in str(e)))

# script
script = myStudy.getPythonScript()
print(script)
exec(script)

# model definition clash bug
x1 = persalys.Input("x1")
x2 = persalys.Input("x2")
code1 = "def _exec(x1, x2):\n    y = x1 + x2\n    return y\n"
code2 = "def _exec(x1):\n    y = x1\n    return y\n"
y = persalys.Output("y")
model1 = persalys.PythonPhysicalModel("model1", [x1, x2], [y], code1)
model2 = persalys.PythonPhysicalModel("model1", [x1, x2], [y], code2)
f1 = model1.getFunction()
y1 = f1([1.0, 2.0])
f2 = model2.getFunction()
y2 = f2([1.0])
y1 = f1([2.0, 3.0])



#!/usr/bin/env python

import openturns as ot
import persalys

Study_0 = persalys.Study("Study_0")
persalys.Study.Add(Study_0)

# variables
dist_z0 = ot.Uniform(100, 150)
z0 = persalys.Input("z0", 100, dist_z0, "")
dist_v0 = ot.Normal(55, 10)
v0 = persalys.Input("v0", 55, dist_v0, "")
dist_m = ot.Normal(80, 8)
m = persalys.Input("m", 80, dist_m, "")
dist_c = ot.Uniform(0, 30)
c = persalys.Input("c", 16, dist_c, "")
z1 = persalys.Output("z1", "")
z2 = persalys.Output("z2", "fake output")
inputs = [z0, v0, m, c]
outputs = [z1, z2]

# mesh model
meshModel = persalys.GridMeshModel(ot.Interval(0.0, 12.0), [20])

# Python model
code = "from math import exp\n\n"
code += "def _exec(z0,v0,m,c):\n    g = 9.81\n    zmin = 0.\n    tau = m / c\n    vinf = -m * g / c\n\n"
code += "    # mesh nodes\n    t = getMesh().getVertices()\n\n"
code += "    z = [max(z0 + vinf * t_i[0] + tau * (v0 - vinf) * (1 - exp(-t_i[0] / tau)), zmin) for t_i in t]\n"
code += "    z2 = [2*max(z0 + vinf * t_i[0] + tau * (v0 - vinf) * (1 - exp(-t_i[0] / tau)), zmin) for t_i in t]\n\n"
code += "    return z, z2"
PhysicalModel_1 = persalys.PythonFieldModel(
    "PhysicalModel_1", meshModel, inputs, outputs, code
)
Study_0.add(PhysicalModel_1)

# central tendency
mcAnalysis1 = persalys.FieldMonteCarloAnalysis("mcAnalysis1", PhysicalModel_1)
mcAnalysis1.setMaximumCalls(30)
Study_0.add(mcAnalysis1)
print(mcAnalysis1)

mcAnalysis1.run()
result1 = mcAnalysis1.getResult()
print("result=", result1)
print(
    "eigen values=",
    result1.getKarhunenLoeveResults()[0].getEigenvalues(),
    result1.getKarhunenLoeveResults()[1].getEigenvalues(),
)

PhysicalModel_1.setParallel(True)
c.setStochastic(False)
mcAnalysis2 = persalys.FieldMonteCarloAnalysis("mcAnalysis2", PhysicalModel_1)
mcAnalysis2.setMaximumCalls(100)
mcAnalysis2.setMaximumElapsedTime(50)
mcAnalysis2.setBlockSize(10)
mcAnalysis2.setInterestVariables(["z2"])
Study_0.add(mcAnalysis2)
print(mcAnalysis2)

mcAnalysis2.run()
result2 = mcAnalysis2.getResult()
print("eigen values=", result2.getKarhunenLoeveResults()[0].getEigenvalues())

# export z evaluation in csv
processSample = mcAnalysis1.getResult().getProcessSample()

fieldSample = ot.Sample(0, meshModel.getMesh().getVertices().getSize())
for field in processSample:
    fieldSample.add(field.getMarginal(1).asPoint())
fieldSample.exportToCSVFile("zDoe.csv")

# import DoE in DataFieldModel
dataModel = persalys.DataFieldModel("dataModel", meshModel)
dataModel.importProcessSample("zDoe.csv", persalys.Tools.Columns)
Study_0.add(dataModel)

analysis3 = persalys.FieldKarhunenLoeveAnalysis("analysis3", dataModel)
analysis3.run()
result3 = analysis3.getResult()
print(result1.getXiSamples()[0] == result3.getXiSamples()[0])
Study_0.add(analysis3)

# script
script = Study_0.getPythonScript()
print(script)
exec(script)



#!/usr/bin/env python

import openturns as ot
import openturns.testing
import persalys

myStudy = persalys.Study("myStudy")

# Model
dist_Q = ot.Normal(10200, 100)
dist_E = ot.Normal(3000, 15)
dist_C = ot.Normal(4000, 60)

Q = persalys.Input("Q", 10200, dist_Q, "Primary energy (W.h)")
E = persalys.Input("E", 3000, dist_E, "Produced electric energy (W.h)")
C = persalys.Input("C", 4000, dist_C, "Valued thermal energy (W.h)")
Ep = persalys.Output("Ep", "Primary energy savings (W.h)")

model = persalys.SymbolicPhysicalModel(
    "myPhysicalModel", [Q, E, C], [Ep], ["1-(Q/((E/((1-0.05)*0.54))+(C/0.8)))"]
)
myStudy.add(model)

outputSample = [
    [0.0600385],
    [0.0812696],
    [0.0684305],
    [0.0892884],
    [0.0292232],
    [0.0511503],
    [0.0378903],
    [0.059432],
]

# Design of Experiment - Parametric analysis ##
bounds = ot.Interval([10035.5, 2975.33, 3901.31], [10364.5, 3024.67, 4098.69])
values = [[bounds.getLowerBound()[i], bounds.getUpperBound()[i]] for i in range(3)]
aDesign = persalys.GridDesignOfExperiment("aDesign_0", model, values)
myStudy.add(aDesign)
aDesign.run()

# Comparaison
openturns.testing.assert_almost_equal(
    outputSample, aDesign.getResult().getDesignOfExperiment().getOutputSample(), 1e-5
)

# Taylor Expansions ##
taylorExpansionsMoments = persalys.TaylorExpansionMomentsAnalysis(
    "myTaylorExpansionMoments", model
)
myStudy.add(taylorExpansionsMoments)
taylorExpansionsMoments.run()
taylorExpansionsMomentsResult = taylorExpansionsMoments.getResult()

# Comparaison
openturns.testing.assert_almost_equal(
    0.059730458221, taylorExpansionsMomentsResult.getMeanFirstOrder()[0], 1e-13
)

# Monte Carlo ##
montecarlo = persalys.MonteCarloAnalysis("myMonteCarlo", model)
montecarlo.setMaximumCalls(1000)
montecarlo.setMaximumCoefficientOfVariation(-1)
myStudy.add(montecarlo)
montecarlo.run()
montecarloResult = montecarlo.getResult()

# Comparaison
openturns.testing.assert_almost_equal(
    0.0597109963361, montecarloResult.getMean()[3][0], 1e-13
)
openturns.testing.assert_almost_equal(
    0.0114128746587, montecarloResult.getStandardDeviation()[3][0], 1e-13
)

meanCI = montecarloResult.getMeanConfidenceInterval()
openturns.testing.assert_almost_equal(0.0590036320343, meanCI.getLowerBound()[3], 1e-13)
openturns.testing.assert_almost_equal(0.0604183606379, meanCI.getUpperBound()[3], 1e-13)
stdCi = montecarloResult.getStdConfidenceInterval()
openturns.testing.assert_almost_equal(0.0109336748621, stdCi.getLowerBound()[3], 1e-13)
openturns.testing.assert_almost_equal(0.0119363302339, stdCi.getUpperBound()[3], 1e-13)

# Sobol ##
sobol = persalys.SobolAnalysis("mySobol", model)
sobol.setReplicationSize(200)
sobol.setMaximumCalls(1000)
myStudy.add(sobol)
sobol.run()
sobolResult = sobol.getResult()

# Comparaison
firstOrderIndicesValues = [[0.643987, 0.0183602, 0.255834]]
totalIndicesValues = [[0.610267, 0.0494237, 0.280706]]
openturns.testing.assert_almost_equal(
    firstOrderIndicesValues, sobolResult.getFirstOrderIndices(), 1e-6
)
openturns.testing.assert_almost_equal(
    totalIndicesValues, sobolResult.getTotalIndices(), 1e-6
)

# SRC ##
src = persalys.SRCAnalysis("mySRC", model)
myStudy.add(src)
src.run()
srcResult = src.getResult()

# Comparaison
openturns.testing.assert_almost_equal(0.628946, srcResult.getIndices()[0][0], 3e-2)
openturns.testing.assert_almost_equal(0.0476118, srcResult.getIndices()[0][1], 3e-2)
openturns.testing.assert_almost_equal(0.318226, srcResult.getIndices()[0][2], 3e-2)

# Chaos ##
values = [
    [
        10035.5,
        10072.1,
        10108.6,
        10145.2,
        10181.7,
        10218.3,
        10254.8,
        10291.4,
        10327.9,
        10364.5,
    ],
    [
        2975.33,
        2980.81,
        2986.29,
        2991.78,
        2997.26,
        3002.74,
        3008.22,
        3013.71,
        3019.19,
        3024.67,
    ],
    [
        3901.31,
        3923.24,
        3945.17,
        3967.1,
        3989.03,
        4010.97,
        4032.9,
        4054.83,
        4076.76,
        4098.69,
    ],
]

design_1 = persalys.GridDesignOfExperiment("aDesign_1", model, values)
design_1.run()
myStudy.add(design_1)

chaos = persalys.FunctionalChaosAnalysis("chaos_0", design_1)
chaos.setChaosDegree(2)
chaos.setSparseChaos(False)
myStudy.add(chaos)
chaos.run()
chaosResult = chaos.getResult()

# Comparaison
openturns.testing.assert_almost_equal(chaosResult.getSobolResult().getFirstOrderIndices()[0], [0.63572, 0.0480642, 0.316216])
openturns.testing.assert_almost_equal(chaosResult.getSobolResult().getTotalIndices()[0], [0.63572, 0.0480642, 0.316216])

# script
script = myStudy.getPythonScript()
print(script)
exec(script)



#! /usr/bin/env python

import openturns as ot
import openturns.testing
import persalys
import os

ot.RandomGenerator.SetSeed(0)

anOTStudy = persalys.Study("anOTStudy")

# Model
X0 = persalys.Input("X0", 1, ot.Normal())
X1 = persalys.Input("X1", 2)
X2 = persalys.Input("X2", 2)
X3 = persalys.Input("X3", 2)
Y0 = persalys.Output("Y0")
Y1 = persalys.Output("Y1")
Y2 = persalys.Output("Y2")

model = persalys.SymbolicPhysicalModel(
    "aModelPhys", [X0, X1, X2, X3], [Y0, Y1, Y2], ["sin(X0)+8*X1"] * 3
)
anOTStudy.add(model)

# Observations ##
filename = "obs_normal2.csv"
sample = ot.Normal(8).getSample(10)
sample.exportToCSVFile(filename)

obs = persalys.Observations(
    "obs", model, filename, [7, 3], [6, 2], ["X3", "X1"], ["Y2", "Y1"]
)
anOTStudy.add(obs)
print("obs=", obs)

openturns.testing.assert_almost_equal(
    obs.getSample(), sample.getMarginal([3, 7, 2, 6]), 1e-16
)

# - reload file
sample2 = sample * 3
sample2.exportToCSVFile(filename)
obs.setFileName(filename)
openturns.testing.assert_almost_equal(
    obs.getSample(), sample2.getMarginal([3, 7, 2, 6]), 1e-16
)

# - change columns
obs.setColumns([4, 5], ["X3", "X1"], [3, 0], ["Y2", "Y1"])
openturns.testing.assert_almost_equal(
    obs.getSample(), sample2.getMarginal([5, 4, 0, 3]), 1e-16
)

try:
    obs.setColumns([4, 5, 0], ["X3", "X1"], [3, 0], ["Y2", "Y1"])
except Exception as e:
    print(
        "InvalidArgumentException occurred: %s" % ("InvalidArgumentException" in str(e))
    )

try:
    obs.setColumns([4, 5, 0], ["X0", "X1", "X2"], [], [])
except Exception as e:
    print(
        "InvalidArgumentException occurred: %s"
        % (
            "InvalidArgumentException : Define observations for at least an output"
            in str(e)
        )
    )

obs.setColumns([4, 5, 1], ["X0", "X1", "X2"], [3, 0], ["Y1", "Y2"])
assert obs.getSample().getDescription() == ["X0", "X1", "X2", "Y1", "Y2"]

# - change observed variables
obs.setColumns([4, 5, 1], ["X3", "X2", "X1"], [3, 0], ["Y1", "Y0"])
assert obs.getSample().getDescription() == ["X1", "X2", "X3", "Y0", "Y1"]

# Observations ##
inS = sample.getMarginal([7, 3])
inS.setDescription(["X3", "X1"])
outS = sample.getMarginal([6, 2])
outS.setDescription(["Y2", "Y1"])
obs2 = persalys.Observations("obs2", model, inS, outS)
anOTStudy.add(obs2)
print("obs2=", obs2)

openturns.testing.assert_almost_equal(
    obs2.getSample(), sample.getMarginal([3, 7, 2, 6]), 1e-16
)

# script
script = anOTStudy.getPythonScript()
print(script)
exec(script)

os.remove(filename)



#! /usr/bin/env python

import openturns as ot
import persalys

myStudy = persalys.Study("myStudy")

# Model
X0 = persalys.Input("X0", ot.Normal(1, 1))
X1 = persalys.Input("X1", ot.Normal(1, 1))
Y0 = persalys.Output("Y0")

model = persalys.SymbolicPhysicalModel("aModelPhys", [X0, X1], [Y0], ["sin(X0) + 8*X1"])
myStudy.add(model)

# limit state ##
limitState = persalys.LimitState("aLimitState", model, "Y0", ot.Greater(), 20.0)
myStudy.add(limitState)

# SORM ##
analysis = persalys.SORMAnalysis("mySORM", limitState)
myStudy.add(analysis)
print(analysis)

analysis.run()

print("result=", analysis.getResult().getSORMResult())

# SORM ##
X2 = persalys.Input("X2", 2)
model.addInput(X2)
model.setFormula("Y0", "sin(X0) + 8*X1 + X2")

analysis2 = persalys.SORMAnalysis("mySORM2", limitState)
myStudy.add(analysis2)
print(analysis2)

analysis2.run()

result2 = analysis2.getResult().getSORMResult()
print("result=", result2)

# SORM ##
analysis3 = persalys.SORMAnalysis("mySORM3", limitState)
analysis3.setPhysicalStartingPoint([1.08161, 2.38966])
analysis3.setOptimizationAlgorithm(ot.AbdoRackwitz())
myStudy.add(analysis3)
print(analysis3)

analysis3.run()

result3 = analysis3.getResult().getSORMResult()
print("result=", result3)

# script
script = myStudy.getPythonScript()
print(script)
exec(script)



#!/usr/bin/env python

import openturns as ot
import persalys

# proba
dist_X0 = ot.Normal(0, 1)
X0 = persalys.Input("X0", 0, dist_X0, "")
dist_X1 = ot.Normal(0, 1)
X1 = persalys.Input("X1", 0, dist_X1, "")
X2 = persalys.Input("X2", 0, "")
X3 = persalys.Input("X3", 0, "")
Y0 = persalys.Output("Y0", "")
inputs = [X0, X1, X2, X3]
outputs = [Y0]
formulas = ["X0+X1+X2+X3"]
SymbolicModel_0 = persalys.SymbolicPhysicalModel(
    "SymbolicModel_0", inputs, outputs, formulas
)
values = [[0], [0], [-0.1, 0.1], [-0.1, 0.1]]
design_0 = persalys.GridDesignOfExperiment("design_0", SymbolicModel_0, values)
design_0.setBlockSize(1)
interestVariables = ["Y0"]
design_0.setInterestVariables(interestVariables)
design_0.run()
metaModel_0 = persalys.FunctionalChaosAnalysis("metaModel_0", design_0)
metaModel_0.setChaosDegree(1)
metaModel_0.setSparseChaos(False)
metaModel_0.setAnalyticalValidation(True)
metaModel_0.setTestSampleValidation(False)
metaModel_0.setKFoldValidation(False)
metaModel_0.run()
print(metaModel_0.getResult())

# deterministic/chaos
X0 = persalys.Input("X0", 0, "")
X1 = persalys.Input("X1", 0, "")
X2 = persalys.Input("X2", 0, "")
X3 = persalys.Input("X3", 0, "")
Y0 = persalys.Output("Y0", "")
inputs = [X0, X1, X2, X3]
outputs = [Y0]
formulas = ["X0+X1+X2+X3"]
SymbolicModel_0 = persalys.SymbolicPhysicalModel(
    "SymbolicModel_0", inputs, outputs, formulas
)
values = [[0], [0], [-0.1, 0.1], [-0.1, 0.1]]
design_0 = persalys.GridDesignOfExperiment("design_0", SymbolicModel_0, values)
design_0.setBlockSize(1)
interestVariables = ["Y0"]
design_0.setInterestVariables(interestVariables)
design_0.run()
metaModel_0 = persalys.FunctionalChaosAnalysis("metaModel_0", design_0)
metaModel_0.setChaosDegree(1)
metaModel_0.setSparseChaos(False)
metaModel_0.setAnalyticalValidation(True)
metaModel_0.setTestSampleValidation(False)
metaModel_0.setKFoldValidation(False)
metaModel_0.run()
print(metaModel_0.getResult())

# deterministic/kriging
X0 = persalys.Input("X0", 0, "")
X1 = persalys.Input("X1", 0, "")
X2 = persalys.Input("X2", 0, "")
X3 = persalys.Input("X3", 0, "")
Y0 = persalys.Output("Y0", "")
inputs = [X0, X1, X2, X3]
outputs = [Y0]
formulas = ["X0+X1+X2+X3"]
SymbolicModel_0 = persalys.SymbolicPhysicalModel(
    "SymbolicModel_0", inputs, outputs, formulas
)
values = [[0], [0], [-0.1, 0.1], [-0.1, 0.1]]
design_0 = persalys.GridDesignOfExperiment("design_0", SymbolicModel_0, values)
design_0.setBlockSize(1)
interestVariables = ["Y0"]
design_0.setInterestVariables(interestVariables)
design_0.run()
metaModel_0 = persalys.KrigingAnalysis("metaModel_0", design_0)
metaModel_0.setBasis(ot.ConstantBasisFactory(2).build())
metaModel_0.setCovarianceModel(ot.SquaredExponential(2))
metaModel_0.setAnalyticalValidation(True)
metaModel_0.setTestSampleValidation(False)
metaModel_0.setKFoldValidation(False)
metaModel_0.run()
print(metaModel_0.getResult())



#! /usr/bin/env python

import openturns as ot
import persalys

# Model with error
X0 = persalys.Input("X0", 1, ot.Normal())
X1 = persalys.Input("X1", 2)
Y0 = persalys.Output("Y0")
Y1 = persalys.Output("Y1")
model2 = persalys.PythonPhysicalModel(
    "model2",
    [X0, X1],
    [Y0],
    "from math import pi\n\ndef _exec(X0, X1):\n    Y0 = 2*X1 - 1/X0\n    return Y0\n",
)
model2.setParallel(True)

design5 = persalys.FixedDesignOfExperiment("design5", model2)
design5.setOriginalInputSample([[0.9, 1.1], [1.8, 2.2], [0, 2.2]])
design5.setBlockSize(3)
design5.run()
assert (
    design5.getResult().getDesignOfExperiment().getOutputSample().getSize() == 2
), "whole block must no fail"


model3 = persalys.PythonPhysicalModel(
    "model3",
    [X0, X1],
    [Y0, Y1],
    "from math import pi\n\ndef _exec(X0, X1):\n    Y0 = 2*X1 - 1/X0\n    Y1=5\n    return Y0, Y1\n",
)
model3.setParallel(True)

# same with multiple outputs
design6 = persalys.FixedDesignOfExperiment("design6", model3)
design6.setInterestVariables(["Y0"])
design6.setOriginalInputSample([[0.9, 1.1], [1.8, 2.2], [0, 2.2]])
design6.setBlockSize(3)
design6.run()
assert (
    design6.getResult().getDesignOfExperiment().getOutputSample().getSize() == 2
), "whole block must no fail"



#! /usr/bin/env python

import openturns as ot
import persalys

myStudy = persalys.Study("myStudy")

# Model
x0 = persalys.Input("x0", ot.Normal())
x1 = persalys.Input("x1", ot.Normal())
y00 = persalys.Output("fake_y0")
y00.setIsSelected(False)
y0 = persalys.Output("y0")

formula_y00 = "x0"
formula_y0 = "10+3*x0+x1"
model = persalys.SymbolicPhysicalModel(
    "aModel", [x0, x1], [y00, y0], [formula_y00, formula_y0]
)
myStudy.add(model)

# Sobol ##
analysis = persalys.SobolAnalysis("aSobol", model)
analysis.setMaximumCalls(1000)
analysis.setMaximumConfidenceIntervalLength(-1)
analysis.setMaximumElapsedTime(30)
analysis.setSeed(2)
analysis.setReplicationSize(250)
analysis.setBlockSize(4)
myStudy.add(analysis)
print(analysis)

analysis.run()

print("result=", analysis.getResult())

# Sobol ##
analysis2 = persalys.SobolAnalysis("aSobol2", model)
analysis2.setMaximumConfidenceIntervalLength(0.2)
analysis2.setMaximumElapsedTime(100000)
myStudy.add(analysis2)
print(analysis2)

analysis2.run()

print("result=", analysis2.getResult())

# Sobol ##
X2 = persalys.Input("x2", 10)
model.addInput(X2)
model.addOutput(persalys.Output("y1"))
model.setFormula("y1", "3*x0 + x1 + x2")

analysis3 = persalys.SobolAnalysis("aSobol3", model)
analysis3.setMaximumCalls(1000)
analysis3.setReplicationSize(200)
analysis3.setBlockSize(4)
analysis3.setSeed(2)
analysis3.setInterestVariables(["y1"])
myStudy.add(analysis3)
print(analysis3)

analysis3.run()

result3 = analysis3.getResult()
print("result=", result3)
persalys.Study.Add(myStudy)
# script
script = myStudy.getPythonScript()
print(script)
exec(script)



#!/usr/bin/env python

import openturns as ot
import openturns.testing
import persalys
import os
import tempfile
import shutil

# ot.Log.Show(ot.Log.DBG)
ot.TBB.Disable()
ot.RandomGenerator.SetSeed(0)
myStudy = persalys.Study("myStudy")

# Model
x1 = persalys.Input("x1", ot.Uniform(0.0, 10.0))
x2 = persalys.Input("x2", ot.Uniform(0.0, 10.0))
x3 = persalys.Input("x3", 0.5)
y00 = persalys.Output("fake_y0")
y00.setIsSelected(False)
y0 = persalys.Output("y0")

formula = ["0.5*x1 + x2+ 28*x3^2"] * 2
model = persalys.SymbolicPhysicalModel("model", [x1, x2, x3], [y00, y0], formula)
myStudy.add(model)

# Design of Experiment
aDesign = persalys.FixedDesignOfExperiment("design", model)
validationInputSample = ot.LHSExperiment(model.getDistribution(), 10).generate()
inputSample = ot.Sample(validationInputSample)
inputSample.stack(ot.Sample(10, [0.5]))
aDesign.setOriginalInputSample(inputSample)
myStudy.add(aDesign)

aDesign.run()

# LM
analysis = persalys.PolynomialRegressionAnalysis("lm_0", aDesign)
analysis.setDegree(2)
analysis.setInteraction(True)
analysis.setDirection(ot.LinearModelStepwiseAlgorithm.FORWARD)
analysis.setPenalty(persalys.PolynomialRegressionAnalysis.AIC)
myStudy.add(analysis)
print(analysis)

analysis.run()
print(analysis.getResult())
metaModel = analysis.getResult().getResultForVariable("y0").getMetaModel()

openturns.testing.assert_almost_equal(
    aDesign.getResult().getDesignOfExperiment().getOutputSample(),
    metaModel(validationInputSample),
    3.0e-5,
    3.0e-5,
)

analysis.setStepwise(False)
analysis.run()
print(analysis.getResult())
metaModel = analysis.getResult().getResultForVariable("y0").getMetaModel()

openturns.testing.assert_almost_equal(
    aDesign.getResult().getDesignOfExperiment().getOutputSample(),
    metaModel(validationInputSample),
    3.0e-5,
    3.0e-5,
)

# test export
temp_path = tempfile.mkdtemp()
path_py = os.path.join(temp_path, "metamodel0.py")
analysis.getResult().getMetaModel().exportStandalonePythonScript(path_py)
assert os.path.exists(path_py)
shutil.rmtree(temp_path)

# boston price model
datamodel = persalys.DataModel(
    "datamodel", "Housing-prices-Boston.csv", range(13), [13]
)
myStudy.add(datamodel)
analysis2 = persalys.PolynomialRegressionAnalysis("lm_1", datamodel)
analysis2.setDegree(2)
analysis2.setInteraction(True)
myStudy.add(analysis2)
analysis2.run()
print(analysis2.getResult())
metaModel2 = analysis2.getResult().getResultForVariable("MEDV").getMetaModel()
f = ot.SymbolicFunction(
    datamodel.getInputSample().getDescription(), analysis2.getResult().getFormulas()
)
x = [1.5] * 13
openturns.testing.assert_almost_equal(metaModel2(x), f(x))

# script
script = myStudy.getPythonScript()
print(script)
exec(script)



#! /usr/bin/env python

import openturns as ot
import openturns.testing
import persalys
import os

myStudy = persalys.Study("myStudy")

# Model
filename = "data1.csv"
ot.RandomGenerator.SetSeed(0)
sample = ot.Normal(3).getSample(300)
sample.stack(ot.Gumbel().getSample(300))
sample.setDescription(["X0", "X1", "X2", "X3"])
sample.exportToCSVFile(filename, ",")
columns = [0, 2, 3]

model = persalys.DataModel("myDataModel", "data1.csv", columns)
myStudy.add(model)
print(model)

# Inference analysis - Kolmogorov##
analysis = persalys.InferenceAnalysis("analysis", model)
variables = ["X0", "X3"]
analysis.setInterestVariables(variables)
factories = [ot.NormalFactory(), ot.GumbelFactory()]
analysis.setDistributionsFactories("X3", factories)
analysis.setLevel(0.1)
analysis.setEstimateParametersConfidenceInterval(True)
myStudy.add(analysis)
print(analysis)

analysis.run()

result = analysis.getResult()
print("result=", result)
print(result.getFittingTestResultForVariable("X3"))

# Inference analysis - Lilliefors##
analysis2 = persalys.InferenceAnalysis("analysis2", model)
variables = ["X0", "X3"]
analysis2.setInterestVariables(variables)
factories = [ot.NormalFactory(), ot.GumbelFactory()]
analysis2.setDistributionsFactories("X3", factories)
analysis2.setLevel(0.1)
analysis2.setTestType(persalys.InferenceAnalysis.Lilliefors)
analysis2.setLillieforsPrecision(0.05)
analysis2.setEstimateParametersConfidenceInterval(True)
myStudy.add(analysis2)
print(analysis2)

analysis2.run()

result = analysis2.getResult()
print("result=", result)
print(result.getFittingTestResultForVariable("X3"))

# script
script = myStudy.getPythonScript()
print(script)
exec(script)

os.remove(filename)



#!/usr/bin/env python

import openturns as ot
import persalys

myStudy = persalys.Study("myStudy")

# mesh points are on the first line
meshModel = persalys.GridMeshModel(
    [persalys.Variable("t", "date")], ot.Interval([1], [12]), [12])
print(meshModel.getMesh().getVertices())

model = persalys.DataFieldModel("myModel", meshModel)
model.importProcessSample("elNino.csv", persalys.Tools.Columns)

myStudy.add(model)
# check that the processSample can be properly converted (used for gui init)
print(model.getProcessSampleAsSample())

analysis1 = persalys.FieldKarhunenLoeveAnalysis("myAnalysis", model)
analysis1.run()
print(analysis1)

myStudy.add(analysis1)

result1 = analysis1.getResult()
print(result1)

# script
script = myStudy.getPythonScript()
print(script)
exec(script)



#!/usr/bin/python3
import time


def main():
    time.sleep(2)
    F = 1.00e00
    E = 3.00e04
    L = 5.00e00
    II = 1.10e-03
    dev = F * L**3 / (3 * E * II)
    with open("beam_output.txt", "w") as f:
        f.write("deviation=" + str(dev) + "\n")
    return dev


if __name__ == "__main__":
    main()



#! /usr/bin/env python

import openturns as ot
import openturns.testing
import persalys
import os

myStudy = persalys.Study("myStudy")

# Model
filename = "data2.csv"
ot.RandomGenerator.SetSeed(0)
R = ot.CorrelationMatrix(4)
R[0, 3] = 0.9
distribution = ot.JointDistribution([ot.Normal(), ot.Normal(), ot.Normal(), ot.Gumbel()], ot.NormalCopula(R))
sample = distribution.getSample(300)
sample.setDescription(["X0", "X1", "X2", "X3"])
sample.exportToCSVFile(filename, ",")
columns = [0, 2, 3]

model = persalys.DataModel("myDataModel", "data2.csv", columns)
myStudy.add(model)
print(model)

# Dependencies inference analysis ##
analysis = persalys.CopulaInferenceAnalysis("analysis", model)
variables = ["X0", "X3"]
factories = [ot.ClaytonCopulaFactory(),
             ot.FrankCopulaFactory(),
             ot.GumbelCopulaFactory(),
             ot.NormalCopulaFactory(),
             ot.StudentCopulaFactory()]
analysis.setDistributionsFactories(variables, factories)
myStudy.add(analysis)
print(analysis)

analysis.run()

result = analysis.getResult()
inferenceSet = result.getCopulaInferenceSetResult(variables)
print(inferenceSet.getBICResults())

# script
script = myStudy.getPythonScript()
print(script)
exec(script)

os.remove(filename)



#!/usr/bin/env python

import openturns as ot
import openturns.testing as ott
import persalys

# ot.Log.Show(ot.Log.DBG)
ot.TBB.Disable()
ot.RandomGenerator.SetSeed(0)
myStudy = persalys.Study("myStudy")

# Model
x1 = persalys.Input("x1", ot.Uniform(0.0, 10.0))
x2 = persalys.Input("x2", ot.Uniform(0.0, 10.0))
x3 = persalys.Input("x3", 0.5)
y00 = persalys.Output("fake_y0")
y00.setIsSelected(False)
y0 = persalys.Output("y0")

formula = ["0.5*x1 + x2+ 28*x3^2"] * 2
model = persalys.SymbolicPhysicalModel("model", [x1, x2, x3], [y00, y0], formula)
myStudy.add(model)

# Design of Experiment
aDesign = persalys.FixedDesignOfExperiment("design", model)
validationInputSample = ot.LHSExperiment(model.getDistribution(), 10).generate()
inputSample = ot.Sample(validationInputSample)
inputSample.stack(ot.Sample(10, [0.5]))
aDesign.setOriginalInputSample(inputSample)
myStudy.add(aDesign)

aDesign.run()

# LM
analysis = persalys.PolynomialRegressionAnalysis("lm_0", aDesign)
analysis.setDegree(2)
analysis.setInteraction(True)
myStudy.add(analysis)

analysis.run()
metaModel = analysis.getResult().getMetaModel().getFunction()

# boston price model
datamodel = persalys.DataModel(
    "datamodel", "Housing-prices-Boston.csv", range(13), [13]
)
myStudy.add(datamodel)
analysis2 = persalys.PolynomialRegressionAnalysis("lm_1", datamodel)
analysis2.setDegree(2)
analysis2.setInteraction(True)
myStudy.add(analysis2)
analysis2.run()
metaModel2 = analysis2.getResult().getMetaModel().getFunction()

# python metamodel
persalys.Study.Add(myStudy)
print("Study instances: ", persalys.Study.GetInstances(), sep='')
pythonMetaModel = analysis.asPythonPhysicalModel(myStudy).getFunction()
ott.assert_almost_equal(pythonMetaModel(validationInputSample), metaModel(validationInputSample))

pythonMetaModel2 = analysis2.asPythonPhysicalModel(myStudy).getFunction()
x = [1.5] * 13
ott.assert_almost_equal(pythonMetaModel2(x), metaModel2(x))



#! /usr/bin/env python

import openturns as ot
import persalys

myStudy = persalys.Study("myStudy")

# Model
X0 = persalys.Input("X0", 1, ot.Normal(), "aDescription")
X1 = persalys.Input("X1", 2)
Y0 = persalys.Output("Y0", "output_1")
inputs = [X0, X1]
outputs = [Y0]
formulas = ["sin(X0)+8*X1"]
model = persalys.SymbolicPhysicalModel("aModelPhys", inputs, outputs, formulas)
myStudy.add(model)
print(model)

# get attributs values
print("function=", model.getFunction())
print("formulas=", model.getFormulas())
print("formula_Y0=", model.getFormula("Y0"))
print("stochastic var=", model.getStochasticInputNames())
print("distribution=", model.getDistribution())
print("copula=", model.getCopula())
print("output_YO=", model.getOutputByName("Y0"))
print("input_XO=", model.getInputByName("X0"))
print("inputs names=", model.getInputNames())
print("outputs names=", model.getOutputNames())
print("hasY0", model.hasOutputNamed("Y0"))
print("hasX0", model.hasInputNamed("X0"))

# set attributs values
# in
model.setInputs(inputs)
model.setOutputs(outputs)
model.setDistribution("X1", ot.LogNormal())
model.setFiniteDifferenceStep("X1", 1e-5)
R = ot.CorrelationMatrix(2)
R[0, 1] = 0.25
model.setCopula(["X0", "X1"], ot.NormalCopula(R))
print("inputs=", model.getInputs())
print("stochastic var=", model.getStochasticInputNames())
print("distribution=", model.getDistribution())
print("copula=", model.getCopula())
# out
model.setFormulas(["sin(X0)+8*X1+0.5"])
print("outputs=", model.getOutputs())

# add variables
# in
X2 = persalys.Input("X2", 10)
model.addInput(X2)
model.setInputName("X2", "X_2")
print("inputs=", model.getInputs())
print("copula=", model.getCopula())
X3 = persalys.Input("X3", 10, "aDescription")
model.addInput(X3)
X4 = persalys.Input("X4", ot.Normal())
model.addInput(X4)
X5 = persalys.Input("X5", ot.Normal(), "aDescription")
model.addInput(X5)
print("stochastic var=", model.getStochasticInputNames())
print("distribution=", model.getDistribution())
print("copula=", model.getCopula())
# out
model.addOutput(persalys.Output("Y1"))
model.setFormula("Y1", "sin(X0)+8*X1+X2")
model.setOutputName("Y1", "Y_1")
model.selectOutput("Y_1", False)
print("formula_Y1=", model.getFormula("Y_1"))
print("outputs=", model.getOutputs())
print("selected outputs=", model.getSelectedOutputsNames())

# remove variables
model.removeInput("X_2")
model.removeInput("X3")
model.removeOutput("Y_1")
print("inputs=", model.getInputs())
print("outputs=", model.getOutputs())

# script
script = model.getPythonScript()
print("script=", script)
exec(script)



#!/usr/bin/env python

import openturns as ot
import persalys

Study_0 = persalys.Study('Study_0')
persalys.Study.Add(Study_0)

# variables
dist_z0 = ot.Uniform(100, 150)
z0 = persalys.Input('z0', 100, dist_z0, '')
dist_v0 = ot.Normal(55, 10)
v0 = persalys.Input('v0', 55, dist_v0, '')
dist_m = ot.Normal(80, 8)
m = persalys.Input('m', 80, dist_m, '')
dist_c = ot.Uniform(0, 30)
c = persalys.Input('c', 16, dist_c, '')
z = persalys.Output('z', '')
z2 = persalys.Output('z2', 'fake output')
inputs = [z0, v0, m, c]
outputs = [z, z2]

# mesh model
meshModel = persalys.GridMeshModel(ot.Interval(0., 12.), [10])

# Python model
code = 'from math import exp\n\ndef _exec(z0,v0,m,c):\n    g = 9.81\n    zmin = 0.\n    tau = m / c\n    vinf = -m * g / c\n\n    # mesh nodes\n    t = getMesh().getVertices()\n\n    z = [max(z0 + vinf * t_i[0] + tau * (v0 - vinf) * (1 - exp(-t_i[0] / tau)), zmin) for t_i in t]\n    z2 = [ t_i[0] for t_i in t]\n\n    return z, z2'
PhysicalModel_1 = persalys.PythonFieldModel('PhysicalModel_1', meshModel, inputs, outputs, code)
Study_0.add(PhysicalModel_1)

# central tendency
mcAnalysis1 = persalys.FieldMonteCarloAnalysis("mcAnalysis", PhysicalModel_1)
mcAnalysis1.setMaximumCalls(5)
mcAnalysis1.setMaximumElapsedTime(1000)
mcAnalysis1.setBlockSize(5)
mcAnalysis1.setSeed(2)
mcAnalysis1.setInterestVariables(['z'])
mcAnalysis1.setKarhunenLoeveThreshold(2e-5)
Study_0.add(mcAnalysis1)

# evaluation
evaluation1 = persalys.FieldModelEvaluation("evaluation", PhysicalModel_1)
evaluation1.setInterestVariables(['z'])
Study_0.add(evaluation1)

# evaluation which generate an error
code = 'from math import exp\n\ndef _exec(z0,v0,m,c):\n    g = 9.81\n    zmin = 0.\n    tau = m / c\n    vinf = -m * g / c\n\n    # mesh nodes\n    t = getMesh().getVertices()\n\n    z = [max(z0 + vinf * t_i[0] + tau * (v0 - vinf) * (1 - exp(-t_i[0] / tau)), zmin) for t_i in t]\n    z2 = [ t_i[0] / 0. for t_i in t]\n\n    return z, z2'
PhysicalModel_2 = persalys.PythonFieldModel('PhysicalModel_2', meshModel, inputs, outputs, code)
Study_0.add(PhysicalModel_2)
evaluation2 = persalys.FieldModelEvaluation("evaluation2", PhysicalModel_2)
evaluation2.setInterestVariables(['z2'])
Study_0.add(evaluation2)




import persalys
import openturns as ot

myStudy = persalys.Study("myStudy")

truncated_input = persalys.Input("X0", 0, ot.TruncatedDistribution(ot.Normal(), -1, 1))
user_defined_input = persalys.Input("X1", 2, ot.UserDefined([[1], [2], [3]], [1, 1, 2]))
hist_input = persalys.Input("X2", 1, ot.Histogram(0, [0.5, 1], [1, 0.5]))
normal_input = persalys.Input("X3", 1, ot.Normal(1, 2))
output = persalys.Output("Y0")
inputs = [truncated_input, user_defined_input, hist_input, normal_input]
outputs = [output]
formulas = ["X0+X1+X2+X3"]
model = persalys.SymbolicPhysicalModel("model", inputs, outputs, formulas)
myStudy.add(model)

script = myStudy.getPythonScript()

namespace = {}
exec(script, namespace)

for i in range(len(inputs)):
    assert inputs[i].getDistribution() == namespace["inputs"][i].getDistribution()



#!/usr/bin/env python

import openturns as ot
import openturns.testing
import persalys

myStudy = persalys.Study("myStudy")

# Model
dist_R = ot.LogNormalMuSigma(300.0, 30.0).getDistribution()
dist_F = ot.Normal(75000.0, 5000.0)

R = persalys.Input("R", 300.0, dist_R, "Yield stress (Pa)")
F = persalys.Input("F", 75000.0, dist_F, "Traction force (N)")
G = persalys.Output("G", "Stress difference (Pa)")

code = "from math import pi\n\ndef _exec(R, F):\n    G = R-F/(pi*100.0)\n    return G\n"
model = persalys.PythonPhysicalModel("myPhysicalModel", [R, F], [G], code)
model.setProcessNumber(1)
myStudy.add(model)

f = model.getFunction()
print(f([300.0, 75000.0]))

# Design of Experiment - Parametric analysis ##
bounds = persalys.GridDesignOfExperiment.GetDefaultBounds(model)
values = [[bounds.getLowerBound()[i], bounds.getUpperBound()[i]] for i in range(2)]
aDesign = persalys.GridDesignOfExperiment("design", model, values)
myStudy.add(aDesign)
aDesign.run()

# Taylor Expansion ##
taylorExpansionsMoments = persalys.TaylorExpansionMomentsAnalysis(
    "myTaylorExpansionMoments", model
)
myStudy.add(taylorExpansionsMoments)
taylorExpansionsMoments.run()
taylorExpansionsMomentsResult = taylorExpansionsMoments.getResult()

# Monte Carlo ##
montecarlo = persalys.MonteCarloAnalysis("myMonteCarlo", model)
montecarlo.setMaximumCalls(1000)
myStudy.add(montecarlo)
montecarlo.run()
montecarloResult = montecarlo.getResult()

meanCI = montecarloResult.getMeanConfidenceInterval()
stdCi = montecarloResult.getStdConfidenceInterval()

# Sobol ##
sobol = persalys.SobolAnalysis("mySobol", model)
sobol.setMaximumConfidenceIntervalLength(-1)
sobol.setMaximumCalls(1000)
sobol.setReplicationSize(250)
myStudy.add(sobol)
sobol.run()
sobolResult = sobol.getResult()

# SRC ##
src = persalys.SRCAnalysis("mySRC", model, 1000)
myStudy.add(src)
src.run()
srcResult = src.getResult()

# script
script = myStudy.getPythonScript()
print(script)
exec(script)



#! /usr/bin/env python

import openturns as ot
import persalys

myStudy = persalys.Study("myStudy")

# Model
X0 = persalys.Input("X0", ot.Normal(1, 1))
X1 = persalys.Input("X1", ot.Normal(1, 1))
Y00 = persalys.Output("fake_Y0")
Y00.setIsSelected(False)
Y0 = persalys.Output("Y0")

formula_Y00 = "X0"
formula_Y0 = "3*(X0-5)^2 + 8*(X1+8)^2"
model = persalys.SymbolicPhysicalModel(
    "aModelPhys", [X0, X1], [Y00, Y0], [formula_Y00, formula_Y0]
)
myStudy.add(model)

# Monte Carlo ##
analysis = persalys.OptimizationAnalysis("optim", model, "Cobyla")
analysis.setInterestVariables(["Y0"])
analysis.setStartingPoint([0.5, 0.5])
analysis.addConstraint("Y0<600.0")
interval = ot.Interval([-5, 0], [5, 0])
analysis.setBounds(interval)
analysis.setVariableInputs(["X0"])
analysis.setMaximumCallsNumber(150)
analysis.setMaximumAbsoluteError(1e-6)
analysis.setMaximumRelativeError(1e-6)
analysis.setMaximumResidualError(1e-6)
analysis.setMaximumConstraintError(1e-6)
myStudy.add(analysis)
print(analysis)

analysis.run()

result = analysis.getResult()
# print("result=", result)
print("x=", result.getOptimalPoint())
print("y=", result.getOptimalValue())

# script
script = myStudy.getPythonScript()
print(script)
exec(script)


# Model 2
X2 = persalys.Input("X2", ot.Normal(1, 1))
Y00 = persalys.Output("X2")
code = "def _exec(X0,X1,X2):\n    Y00 = X0\n    Y0 = 3*(X0-5)**2 + 8*(X1+8)**2\n    return Y0,X2"
model2 = persalys.PythonPhysicalModel("pythonModel", [X0, X1, X2], [Y0, Y00], code)
myStudy.add(model2)

analysis = persalys.OptimizationAnalysis("optim", model2, "Cobyla")
analysis.setInterestVariables(["Y0"])
analysis.setStartingPoint([0.5, 0.5, 0])
analysis.addConstraint("Y0<600.0")
interval = ot.Interval([-5, 0, -2], [5, 0, 2])
analysis.setBounds(interval)
analysis.setVariableInputs(["X0"])
analysis.setMaximumCallsNumber(150)
analysis.setMaximumAbsoluteError(1e-6)
analysis.setMaximumRelativeError(1e-6)
analysis.setMaximumResidualError(1e-6)
analysis.setMaximumConstraintError(1e-6)
analysis.run()
cstr = analysis.defineProblem().getInequalityConstraint().getEvaluation().getImplementation()
print(cstr)
point = analysis.getResult().getFinalPoints()
print(cstr(point.getMarginal(cstr.getInputPositions())))
